/*= R&S SMR Microwave Generator =============================================*/
/* LabWindows/CVI 6.0 VXIpnp Instrument Driver                               */
/* Original Release: March 2000                                              */
/* By: Jiri Hula                                                             */
/*                                                                           */
/* Should you have any technical questions please contact the hotline of     */
/* Rohde & Schwarz Vertriebs-GmbH Rohde & Schwarz Support Center             */
/*                                                                           */
/* e-mail: CustomerSupport@rsd.rohde-schwarz.com                             */
/*                                                                           */
/* Modification History:                                                     */
/*                                                                           */
/*   1.2.5, 11/2007, P.Marcalik                                              */ 
/*    New functions:                                                         */
/*          rssmr_setRFLevelIncrement                                        */
/*          rssmr_setRfLevelUnit                                             */
/*          rssmr_getFreqCorrPoints                                          */
/*          rssmr_setALCSlope                                                */
/*          rssmr_setRfBlankingBehaviorSync                                  */
/*          rssmr_setRfBlankingBehavior                                      */
/*          rssmr_setPulseTrigLevel                                          */
/*          rssmr_setLevCalState                                             */
/*          rssmr_getRefOscCalVal                                            */
/*          rssmr_storeRefOscCalVal                                          */
/*          rssmr_setListBlankingBehavior                                    */
/*          rssmr_calibration                                                */
/*          rssmr_utilSystemKeylock                                          */
/*          rssmr_utilSetDisplayUpdate                                       */
/*     Removed functions:                                                    */
/*          rssmr_setSeqMode                                                 */
/*          rssmr_selSequence                                                */
/*          rssmr_deleteSequence                                             */
/*          rssmr_deleteAllSequence                                          */
/*          rssmr_getListOfSequence                                          */
/*          rssmr_setSeqDwellTime                                            */
/*          rssmr_getSeqSpaceProper                                          */
/*          rssmr_recallSequence                                             */
/*          rssmr_getSeqPoints                                               */
/*          rssmr_setMSeqTrigSource                                          */
/*                                                                           */
/*                                                                           */
/*   1.2.4, 07/2007, Jiri Kominek                                            */ 
/*      fixed s/ms problem                                                   */
/*      changesd:                                                            */  
/*          rssmr_configureList                                              */  
/*          rssmr_setListDwellTime                                           */  
/*                                                                           */
/*   1.2.3, 12/2004, Werner Klotzsche                                        */ 
/*      FW 1.29 and FW 1.30, fixed s/ms problem                              */
/*      changesd:                                                            */  
/*          rssmr_configPulseOutput                                          */  
/*          rssmr_setPulseDelay                                              */  
/*          rssmr_setPulsePeriod                                             */  
/*          rssmr_setPulseWidth                                              */                                                                      
/*          rssmr_setDoublePulseDelay                                        */  
/*          rssmr_configRFFreqSweep                                          */  
/*          rssmr_setRFFreqSweepDwellTime                                    */  
/*          rssmr_configRFLevelSweep                                         */   
/*          rssmr_setRFLevelSweepDwellTime                                   */  
/*          rssmr_configLFSweep                                              */  
/*          rssmr_setLFSweepDwellTime                                        */  
/*                                                                           */
/*   1.2.2, 04/2004, Jiri Kominek                                            */ 
/*    Added CHM Help file                                                    */ 
/*    Fixed floating point precision problem                                 */
/*                                                                           */
/*   1.2.1, 01/2004, Werner Klotzsche                                        */ 
/*    Modifications                                                          */ 
/*    For Agilent VISA (Ver. M01.01) and Serial Interface only               */
/*     - changed rssmr_init                                                  */     
/*                                                                           */                                                                           
/*   1.2, 12/2002, Martin Koutny                                             */
/*    New functions:                                                         */
/*     - Get Free Space (rssmr_getFreeCorrTables)                            */
/*     - Set Fixed Power Mode (rssmr_setFixedPowerMode)                      */
/*     - Set Sweep Power Mode (rssmr_setSweepPowerMode)                      */
/*     - Set List Power Mode (rssmr_setListPowerMode)                        */
/*     - Set RF Level Sweep Spacing (rssmr_setRFLevelSweepSpacing)           */
/*    Modifications (descriptions only):                                     */
/*     - Get List Of Correction Lists (rssmr_getListOfCorrLists)             */
/*     - Get List Of Lists (rssmr_getListOfLists)                            */
/*     - Get List Of Sequences (rssmr_getListOfSequences)                    */
/*     - Configure RF Frequency (rssmr_configRFFreq)                         */
/*     - Configure RF Level (rssmr_configRFLevel)                            */
/*    Configure RF Frequency Sweep (rssmr_configRFFreqSweep)                 */
/*      range checked against the current MIN and MAX values                 */
/*    Configure RF Level Sweep (rssmr_configRFLevelSweep)                    */
/*      range checked against the current MIN and MAX values                 */
/*    Configure RS232 (rssmr_utilConfigureRS232)                             */
/*      Added possibility to skip HW handshake settings                      */
/*    Drawback of Agilent VISA is patched by adding extra termination        */
/*      character "\n" to each command                                       */
/*    Included support for SMR20, SMR27, SMR30, SMR40, SMR50 and SMR60       */
/*    Option & instrument model checking is now provided                     */
/*    Range checking for low level RF Frequency and RF Level functions is    */
/*      not provided due to instrument specific dependecies.                 */
/*    Code cleanup & code verification                                       */
/*                                                                           */
/*  1.1, 03/2002, Jan Sima                                                   */
/*     Additions:                                                            */
/*     - rssmr_setRFLevelReset                                               */
/*     Modifications:                                                        */
/*     - in rssmr_init now sending linefeed before first command is sent     */
/*       to the instrument                                                   */
/*                                                                           */
/*     Note:                                                                 */
/*     rssmr_setRFLevelReset and rssmr_setCWFreqReset                        */
/*          are not supported by firmware 1.22 or below                      */
/*                                                                           */
/*===========================================================================*/
#include <visa.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include "rssmr.h"

#define RSSMR_REVISION     "Rev 1.2.5, 11/2007, CVI 8"  /* Instrument driver revision */
#define BUFFER_SIZE         512L                        /* File I/O buffer size */

/*= Macros ==================================================================*/
#define CHECKERR(fCal) if (rssmr_status = (fCal), rssmr_status < VI_SUCCESS) \
    return rssmr_status; else

#define CHECKERR_CU(fCal) if (rssmr_status = (fCal), rssmr_status < VI_SUCCESS) \
    return rssmr_initCleanUp (rmSession, instrSession, rssmr_status); else

/* Check presency of instrument's option (when error checking is enabled) */
#define CHECKOPT(option) if (rssmr_status = (instrPtr -> instrOption & (option)) ? \
    VI_SUCCESS : (instrPtr -> errorChecking) ? RSSMR_ERROR_INSTRUMENT_OPTION : VI_SUCCESS, \
    rssmr_status < VI_SUCCESS) \
    return rssmr_status; else

#define GETFREQMAX(instrModel, freqMax) \
    { \
    if (instrPtr -> errorChecking) \
        switch (instrModel) \
            { \
            case RSSMR_SMR20: \
                freqMax = 20.0e9; \
                break; \
            case RSSMR_SMR27: \
                freqMax = 27.0e9; \
                break; \
            case RSSMR_SMR30: \
                freqMax = 30.0e9; \
                break; \
            case RSSMR_SMR40: \
                freqMax = 40.0e9; \
                break; \
            case RSSMR_SMR50: \
                freqMax = 50.0e9; \
                break; \
            case RSSMR_SMR60: \
                freqMax = 60.0e9; \
                break; \
            default: \
                break; \
            } \
    }

#define IEEE_ERROR_BITS     0x3C

/* Instrument Options */
#define RSSMR_OPT_B1        (1 << 0)
/* reserved                 (1 << 1) */
#define RSSMR_OPT_B3        (1 << 2)
#define RSSMR_OPT_B4        (1 << 3)
#define RSSMR_OPT_B5        (1 << 4)
/* reserved                 (1 << 5) */
#define RSSMR_OPT_B11       (1 << 6)
#define RSSMR_OPT_B14       (1 << 7)
#define RSSMR_OPT_B15       (1 << 8)
#define RSSMR_OPT_B17       (1 << 9)
#define RSSMR_OPT_B18       (1 << 10)
#define RSSMR_OPT_B19       (1 << 11)
#define RSSMR_OPT_B20       (1 << 12)
#define RSSMR_OPT_B23       (1 << 13)
#define RSSMR_OPT_B24       (1 << 14)

/* Instrument Models */
#define RSSMR_SMR20         (1 << 0)
#define RSSMR_SMR27         (1 << 1)
#define RSSMR_SMR30         (1 << 2)
#define RSSMR_SMR40         (1 << 3)
#define RSSMR_SMR50         (1 << 4)
#define RSSMR_SMR60         (1 << 5)

/*****************************************************************************/
/*= INSTRUMENT-DEPENDENT COMMAND ARRAYS =====================================*/
/*****************************************************************************/
static ViString freqResetArr[] = {"INCL", "EXCL"};
static ViString RFFreqModeArr[] = {"FIX", "SWE", "LIST"};  
static ViString SweepModeArr[] = {"AUTO", "MAN", "STEP"};
static ViString sweepBTimeArr[] = {"NORM", "LONG"};
static ViString RFFreqSweepSpacArr[] = {"LIN","LOG","RAMP"};
static ViString RFFreqSweepStepArr[] = {"LIN","LOG"};
static ViString RFAttModeArr[] = {"AUTO", "FIX"};
static ViString RFOutPonSArr[] = {"OFF", "UNCH"};
static ViString AMSourceArr[] = {"INT", "EXT1", "EXT2"};
static ViString AMExtCoupArr[] = {"AC", "DC"};
static ViString AMExtImpArr[] = {"600", "100000"};
static ViString PolarityArr[] = {"NORM", "INV"};
static ViString PMSourceArr[] = {"EXT", "INT"};
static ViString DMTypeArr[] = {"ASK", "FSK"};
static ViString ALCSourArr[] = {"INT","DIOD","RS_NRVS","HP436A"};
static ViString LFFuncArr[] = {"SIN", "SQU"};
static ViString LFFreqSweepSpacArr[] = {"LIN", "LOG"};
static ViString pulseOutSourceArr[] = {"OFF", "PULS", "VID"};
static ViString RFOscSourceArr[] = {"INT", "EXT"};
static ViString ListModeArr[] = {"AUTO", "STEP"};
static ViString TriggerSourceArr[] = {"SING", "EXT", "AUTO"};
static ViString PulseTriggerSourceArr[] = {"AUTO","EXT","EXT_GATED"};
static ViString PulseTriggerSlopeArr[] = {"POS", "NEG"};
static ViString TriggerSlopeArr[] = {"POS", "NEG", "EITH"};
static ViString TriggerArr[] = {"SWE", "LIST", "MSEQ"};
static ViString SystemModeArr[] = {"FIX", "MSEQ"};
static ViString DisplayParamArr[] = {"ALL", "AMPL", "FREQ"};
static ViString BaudRateArr[] = {"1200", "2400", "4800", "9600", "19200", "38400",
                                 "57600", "115200"};
static ViString softHandArr[] = {"XON", "NONE"};
static ViString ParityArr[] = {"ODD","EVEN","NONE"};
static ViString hardHandArr[] = {"ON", "IBF", "RFR"};
static ViString TestArr[] = {"RAM", "ROM", "BATT"};
static ViString RegisterArr[] = {":STAT:OPER:EVEN",":STAT:OPER:COND",
                                 ":STAT:OPER:PTR",":STAT:OPER:NTR",
                                 ":STAT:OPER:ENAB",":STAT:QUES:EVEN",
                                 ":STAT:QUES:COND",":STAT:QUES:PTR",
                                 ":STAT:QUES:NTR",":STAT:QUES:ENAB",
                                 "*ESE","*ESR","*SRE","*STB"};
static ViString RfBlankingArr[] = {"OFF", "ON", "AUTO"};
static ViString ALCslopeArr[] = {"SLOW","MED","FAST"};
static ViString PulseTriggerLevelArr[] = {"TTL","V05","VM25"};
static ViString CalArr[] = {":CAL:ALL",":CAL:LFG"};
static ViString levelUnitArr[] = {"DBM","DBUV","V"};   

/*****************************************************************************/
/*= INSTRUMENT-DEPENDENT STATUS/RANGE STRUCTURE  ============================*/
/*****************************************************************************/
/* rssmr_stringValPair is used in the rssmr_error_message function           */
/* rssmr_statusDataRanges is used to track session dependent status & ranges */
/*===========================================================================*/
typedef struct  rssmr_stringValPair
{
   ViStatus stringVal;
   ViString stringName;
}  rssmr_tStringValPair;
 
struct rssmr_statusDataRanges {

    ViInt32     instrModel;
    ViInt32     instrOption;
    ViBoolean   firmware;
    ViBoolean   errorChecking;
    ViChar      instrDriverRevision[256];
};

typedef struct rssmr_statusDataRanges *rssmr_instrRange;

/*****************************************************************************/
/*= UTILITY ROUTINE DECLARATIONS (Non-Exportable Functions) =================*/
/*****************************************************************************/
ViBoolean rssmr_invalidViBooleanRange (ViBoolean val);
ViBoolean rssmr_invalidViInt32Range (ViInt32 val, ViInt32 min, ViInt32 max);
ViBoolean rssmr_invalidViReal64Range (ViReal64 val, ViReal64 min,
                ViReal64 max);
ViBoolean rssmr_invalidViString (ViString val, ViUInt32 maxLength);
ViStatus rssmr_initCleanUp (ViSession openRMSession,
                ViPSession openInstrSession,
                ViStatus currentStatus);
ViStatus rssmr_defaultInstrSetup (ViSession openInstrSession);

/*****************************************************************************/
/*------ INSERT INSTRUMENT-DEPENDENT UTILITY ROUTINE DECLARATIONS HERE ------*/
/*****************************************************************************/
ViStatus rssmr_instrOptions (ViSession instrSession, ViPInt32 options);
ViStatus rssmr_checkStatus (ViSession instrSession);

/*****************************************************************************/
/*====== USER-CALLABLE FUNCTIONS (Exportable Functions) =====================*/
/*****************************************************************************/

/*===========================================================================*/
/* Function: Initialize                                                      */
/* Purpose:  This function opens the instrument, queries the instrument      */
/*           for its ID, and initializes the instrument to a known state.    */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_init (ViRsrc resourceName, ViBoolean IDQuery,
                    ViBoolean resetDevice, ViPSession instrSession)
{
    ViStatus    rssmr_status = VI_SUCCESS;
    ViUInt16    interface = VI_INTF_GPIB;
    ViSession   rmSession = 0;
    ViChar      rdBuffer[BUFFER_SIZE];

    /*- Check input parameter ranges ----------------------------------------*/
    if (rssmr_invalidViBooleanRange (IDQuery))
        return VI_ERROR_PARAMETER2;
    if (rssmr_invalidViBooleanRange (resetDevice))
        return VI_ERROR_PARAMETER3;

    /*- Open instrument session ---------------------------------------------*/
    CHECKERR( viOpenDefaultRM (&rmSession));

    if ((rssmr_status = viOpen (rmSession, resourceName, VI_LOAD_CONFIG,
                                 VI_NULL, instrSession)) < 0)
        {
        viClose (rmSession);
        return rssmr_status;
        }

    /* Serial interface support */
    CHECKERR_CU( viGetAttribute (*instrSession, VI_ATTR_INTF_TYPE, &interface));

    if (interface == VI_INTF_ASRL)  /* Serial Port */
        {
        CHECKERR_CU( viSetAttribute (*instrSession, VI_ATTR_TERMCHAR, '\n'));
        CHECKERR_CU( viSetAttribute (*instrSession, VI_ATTR_TERMCHAR_EN,
                                     VI_TRUE));
        CHECKERR_CU( viSetAttribute (*instrSession, VI_ATTR_ASRL_END_IN,
                                     VI_ASRL_END_TERMCHAR));
        /* Changed to VI_ASRL_END_NONE due to Agilent VISA */
        CHECKERR_CU( viSetAttribute (*instrSession, VI_ATTR_ASRL_END_OUT, VI_ASRL_END_NONE));
        CHECKERR_CU( viFlush (*instrSession, VI_ASRL_IN_BUF));

        CHECKERR_CU( viPrintf (*instrSession, "\n\n"));
        }
        
    /*- Configure VISA Formatted I/O ----------------------------------------*/
    CHECKERR_CU( viSetAttribute (*instrSession, VI_ATTR_TMO_VALUE, 10000));
    CHECKERR_CU( viSetBuf (*instrSession, VI_READ_BUF|VI_WRITE_BUF, 4000));
    CHECKERR_CU( viSetAttribute (*instrSession, VI_ATTR_WR_BUF_OPER_MODE,
                                 VI_FLUSH_ON_ACCESS));
    CHECKERR_CU( viSetAttribute (*instrSession, VI_ATTR_RD_BUF_OPER_MODE,
                                 VI_FLUSH_ON_ACCESS));

    /*- Identification Query ------------------------------------------------*/
    if (IDQuery)
        {
        CHECKERR_CU( viQueryf (*instrSession, "*IDN?\n", "%s", rdBuffer));
        if (!strstr (rdBuffer, "ROHDE&SCHWARZ,SMR"))
            return rssmr_initCleanUp (rmSession, instrSession,
                                       VI_ERROR_FAIL_ID_QUERY);
        }       

    /*- Reset instrument ----------------------------------------------------*/
    if (resetDevice)
        rssmr_status = rssmr_reset (*instrSession);
    else  /*- Send Default Instrument Setup ---------------------------------*/
        rssmr_status = rssmr_defaultInstrSetup (*instrSession);
    CHECKERR_CU( rssmr_status);

    return rssmr_status;
}

/*===========================================================================*/
/* Function: Output Ampl Modulated Signal                                    */
/* Purpose:  This function configures RF frequency output, switches it on    */
/*           and applies AM modulation on it with selected parameters.       */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_exOutAmpl (ViSession instrSession, ViReal64 frequency,
                                   ViReal64 offset, ViReal64 level,
                                   ViInt32 source, ViReal64 depth,
                                   ViReal64 AMFrequency)
{
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;
    ViReal64    freqMin = 0,
                freqMax = 60.0e9,
                powMin  = -130.0,
                powMax  = 25.0;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr))

    freqMin = ((instrPtr -> instrOption) & RSSMR_OPT_B11) ? 10.0e6 : 1.0e9;
    GETFREQMAX (instrPtr -> instrModel, freqMax);
    powMin = ((instrPtr -> instrOption) & (RSSMR_OPT_B15 | RSSMR_OPT_B17 | RSSMR_OPT_B18)) ? -130.0 : -20.0;

    if (rssmr_invalidViReal64Range (frequency, freqMin, freqMax))     
        return VI_ERROR_PARAMETER2;
    if (rssmr_invalidViReal64Range (offset, -50.0e9, 50.0e9))
        return VI_ERROR_PARAMETER3;
    if (rssmr_invalidViReal64Range (level, powMin, powMax))    
        return VI_ERROR_PARAMETER4;
    if (rssmr_invalidViInt32Range (source,0,2))  
       return VI_ERROR_PARAMETER5;
    if (rssmr_invalidViReal64Range (depth, 0.0, 100.0))  
       return VI_ERROR_PARAMETER6;
    if (rssmr_invalidViReal64Range (AMFrequency, 0.1, 10.0e6))   
       return VI_ERROR_PARAMETER7;

    /* Switches state checking to off. This will speed-up performance of the bus. */
    CHECKERR (rssmr_errorCheckState (instrSession, VI_FALSE));
    
    CHECKERR (rssmr_configRFFreq (instrSession, frequency, offset, 1.0));
    CHECKERR (rssmr_configRFLevel (instrSession, level, 0.0, level));
    CHECKERR (rssmr_configAMModulation (instrSession, source, depth, AMFrequency));
    CHECKERR (rssmr_setRFOutputState (instrSession, VI_TRUE));
    
    /* Switches state checking to on. */
    CHECKERR (rssmr_errorCheckState (instrSession, VI_TRUE));

    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Configure RF Frequency                                          */
/* Purpose:  This function sets the frequency, offset and multiplier.        */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_configRFFreq (ViSession instrSession,
                                      ViReal64 frequency, ViReal64 offset,
                                      ViReal64 multiplier)
{
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;
    ViReal64    freqMin = 0,
                freqMax = 60.0e9;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr))

    freqMin = ((instrPtr -> instrOption) & RSSMR_OPT_B11) ? 10.0e6 : 1.0e9;
    GETFREQMAX (instrPtr -> instrModel, freqMax);

    if (rssmr_invalidViReal64Range (frequency, freqMin, freqMax))     
        return VI_ERROR_PARAMETER2;
    if (rssmr_invalidViReal64Range (offset, -50.0e9, 50.0e9))
        return VI_ERROR_PARAMETER3;
    if (rssmr_invalidViReal64Range (multiplier, 1.0, 10.0))  
        return VI_ERROR_PARAMETER4;

    CHECKERR (viPrintf (instrSession, ":SOUR:FREQ:OFFS 0.0;MULT 1.0;MODE CW;"
                        ":SOUR:FREQ %Lf;:SOUR:FREQ:OFFS %Lf;MULT %Le\n",
                        frequency, offset, multiplier));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set CW Frequency                                                */
/* Purpose:  This function sets the frequency for CW operation               */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setCWFreq (ViSession instrSession, ViReal64 frequency)
{                               
    ViStatus rssmr_status = VI_SUCCESS;

    CHECKERR (viPrintf (instrSession, ":SOUR:FREQ %Lf\n", frequency));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set RF Frequency Offset                                         */
/* Purpose:  This function sets the frequency offset of a mixer.             */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setRFFreqOffset (ViSession instrSession,
                                         ViReal64 offset)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    
    if (rssmr_invalidViReal64Range (offset, -50.0e9, 50.0e9))
        return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:FREQ:OFFS %Lf\n", offset));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}
                                         
/*===========================================================================*/
/* Function: Set RF Frequency Multiplier                                     */
/* Purpose:  This function sets the frequency multiplier.                    */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setRFFreqMultiplier (ViSession instrSession,
                                             ViReal64 multiplier)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    
    if (rssmr_invalidViReal64Range (multiplier, 1.0, 10.0))  
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:FREQ:MULT %Le\n", multiplier));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set CW Freq Reset                                               */
/* Purpose:  This function determines the effect of the recall               */
/*           function on the frequency.                                      */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setCWFreqReset (ViSession instrSession,
                                        ViInt32 resetState)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    
    if (rssmr_invalidViInt32Range (resetState, 0, 1))    
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:FREQ:RCL %s\n", freqResetArr[resetState]));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set RF Frequency Mode                                           */
/* Purpose:  This function specifies the RF operating mode.                  */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setRFFreqMode (ViSession instrSession, ViInt32 mode)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    
    if (rssmr_invalidViInt32Range (mode, 0, 2))  
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:FREQ:MODE %s\n", RFFreqModeArr[mode]));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Configure RF Frequency Sweep                                    */
/* Purpose:  This function sets the sweep mode, start and stop frequency.    */
/*           increment and dwell time.                                       */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_configRFFreqSweep (ViSession instrSession, ViInt32 mode,
                                           ViReal64 startFrequency,
                                           ViReal64 stopFrequency,
                                           ViReal64 increment, ViReal64 dwellTime)
{
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;
    ViReal64    freqMin = 10.0e6,
                freqMax = 60.0e9;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B4);

    /* Query actual MIN and MAX values */
    CHECKERR (viQueryf (instrSession, ":SOUR:FREQ? MIN;FREQ? MAX\n", "%Le;%Le",
        &freqMin, &freqMax));

    if (rssmr_invalidViInt32Range (mode, 0, 2))  
       return VI_ERROR_PARAMETER2;
    if (rssmr_invalidViReal64Range (startFrequency, freqMin, freqMax))
       return VI_ERROR_PARAMETER3;
    if (rssmr_invalidViReal64Range (stopFrequency, freqMin, freqMax))
       return VI_ERROR_PARAMETER4;
    if (rssmr_invalidViReal64Range (increment, 0.0, 10.0e9))     
       return VI_ERROR_PARAMETER5;
    if (rssmr_invalidViReal64Range (dwellTime, 0.01, 5.0))   
       return VI_ERROR_PARAMETER6;                              

    CHECKERR (viPrintf (instrSession, ":SOUR:SWE:FREQ:MODE %s;:SOUR:FREQ:STAR %Lf"
        ";STOP %Lf;:SOUR:FREQ:STEP:INCR %Lf;:SOUR:SWE:DWEL %Lfs\n",
        SweepModeArr[mode], startFrequency, stopFrequency, increment,
        dwellTime));
    CHECKERR (rssmr_checkStatus (instrSession));

    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set RF Freq Sweep Mode                                          */
/* Purpose:  This function specifies specifies the run of the sweep.         */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setRFFreqSweepMode (ViSession instrSession,
                                            ViInt32 mode)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B4);

    if (rssmr_invalidViInt32Range (mode, 0, 2))  
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:SWE:FREQ:MODE %s\n", SweepModeArr[mode]));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set RF Start Frequency                                          */
/* Purpose:  This function defines the starting value of the frequency       */
/*           for the sweep operation.                                        */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setRFStartFreq (ViSession instrSession,
                                        ViReal64 frequency)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B4);

    CHECKERR (viPrintf (instrSession, ":SOUR:FREQ:STAR %Lf\n", frequency));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set RF Stop Frequency                                           */
/* Purpose:  This function defines the final value of the frequency          */
/*           for the sweep operation.                                        */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setRFStopFreq (ViSession instrSession,
                                       ViReal64 frequency)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B4);

    CHECKERR (viPrintf (instrSession, ":SOUR:FREQ:STOP %Lf\n", frequency));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set RF Frequency Increment                                      */
/* Purpose:  This function defines the step width for the frequency setting. */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setRFFreqIncrement (ViSession instrSession,
                                            ViReal64 increment)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B4);

    if (rssmr_invalidViReal64Range (increment, 0.0, 10.0e9))     
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:FREQ:STEP:INCR %Lf\n", increment));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set RF Freq Sweep Dwell Time                                    */
/* Purpose:  This function defines the dwell time per frequency step.        */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setRFFreqSweepDwellTime (ViSession instrSession,
                                                 ViReal64 time)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B4);

    if (rssmr_invalidViReal64Range (time, 0.01, 5.0))    
       return VI_ERROR_PARAMETER2;                              

    CHECKERR (viPrintf (instrSession, ":SOUR:SWE:DWEL %Lfs\n", time));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set Sweep Blank Time                                            */
/* Purpose:  This function defines the blank time of the sweep.              */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setSweepBlankTime (ViSession instrSession,
                                           ViInt32 time)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B4);

    if (rssmr_invalidViInt32Range (time, 0, 1))  
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:SWE:BTIM %s\n", sweepBTimeArr[time]));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set RF Freq Sweep Spacing                                       */
/* Purpose:  This function defines whether the steps have linear             */
/*           or logarithmic spacings.                                        */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setRFFreqSweepSpacing (ViSession instrSession,
                                               ViInt32 spacing)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B4);

    if (rssmr_invalidViInt32Range (spacing, 0, 2))   
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:SWE:FREQ:SPAC %s\n",
        RFFreqSweepSpacArr[spacing]));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set RF Freq Sweep Step                                          */
/* Purpose:  This function defines the step width for linear                 */
/*           and logarithmic sweeps                                          */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setRFFreqSweepStep (ViSession instrSession,
                                            ViInt32 mode, ViReal64 step)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B4);

    if (rssmr_invalidViInt32Range (mode, 0, 2))
       return VI_ERROR_PARAMETER2;

    if (mode == 0)
        {
        if (rssmr_invalidViReal64Range (step, 0.0, 10.0e9))  
           return VI_ERROR_PARAMETER3;
        }
    else
        {
        if (rssmr_invalidViReal64Range (step, 0.01, 100.0))  
           return VI_ERROR_PARAMETER3;
        }
                                                                   
    CHECKERR (viPrintf (instrSession, ":SOUR:SWE:FREQ:STEP:%s %Lf\n",
        RFFreqSweepStepArr[mode], step));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set RF Sweep Frequency Manual                                   */
/* Purpose:  This function the manual frequency.                             */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setRFSweepFreqManual (ViSession instrSession,
                                              ViReal64 frequency)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B4);

    CHECKERR (viPrintf (instrSession, ":SOUR:FREQ:MAN %Lf\n", frequency));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set RF Center Frequency.                                        */
/* Purpose:  This function sets the sweep range by means                     */
/*           of the center frequency.                                        */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setRFCenterFreq (ViSession instrSession,
                                         ViReal64 frequency)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B4);

    CHECKERR (viPrintf (instrSession, ":SOUR:FREQ:CENT %Lf\n", frequency));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set RF Span Frequency.                                          */
/* Purpose:  This function sets the the frequency range for the sweep.       */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setRFSpanFreq (ViSession instrSession,
                                       ViReal64 frequency)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B4);

    CHECKERR (viPrintf (instrSession, ":SOUR:FREQ:SPAN %Lf\n", frequency));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Configure RF Level.                                             */
/* Purpose:  This function sets the RF level, offset and limit.              */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_configRFLevel (ViSession instrSession, ViReal64 level,
                                       ViReal64 offset, ViReal64 limit)
{
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;    
    ViReal64 powMin = -130.0;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr))

    powMin = ((instrPtr -> instrOption) & (RSSMR_OPT_B15 | RSSMR_OPT_B17 | RSSMR_OPT_B18)) ? -130.0 : -20.0;

    if (rssmr_invalidViReal64Range (level, powMin, 25.0))    
       return VI_ERROR_PARAMETER2;
    if (rssmr_invalidViReal64Range (offset, -100.0, 100.0))  
       return VI_ERROR_PARAMETER3;
    if (rssmr_invalidViReal64Range (limit, powMin, 25.0))    
       return VI_ERROR_PARAMETER4;

    CHECKERR (viPrintf (instrSession, ":POW:OFFS 0.0;:POW:LIM MAX;:POW %Lf;"
                        ":POW:OFFS %Lf;:POW:LIM %Lf\n", level, offset, limit));
    CHECKERR (rssmr_checkStatus (instrSession));

    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set RF Level.                                                   */
/* Purpose:  This function sets the RF output level in operating mode CW.    */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setRFLevel (ViSession instrSession, ViReal64 level)
{                               
    ViStatus rssmr_status = VI_SUCCESS;

    CHECKERR (viPrintf (instrSession, ":POW %Lf\n", level));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set RF Level Offset                                             */
/* Purpose:  This function sets the constant level offset.                   */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setRFLevelOffset (ViSession instrSession,
                                          ViReal64 offset)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    
    if (rssmr_invalidViReal64Range (offset, -100.0, 100.0))  
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":POW:OFFS %Lf\n", offset));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set RF Level Limit                                              */
/* Purpose:  This function sets the maximum RF output level.                 */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setRFLevelLimit (ViSession instrSession, ViReal64 limit)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;    
    ViReal64 powMin = -130.0;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr))

    powMin = ((instrPtr -> instrOption) & (RSSMR_OPT_B15 | RSSMR_OPT_B17 | RSSMR_OPT_B18)) ? -130.0 : -20.0;
    
    if (rssmr_invalidViReal64Range (limit, powMin, 25.0))    
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:POW:LIM:AMPL %Lf\n", limit));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set RF Level Reset                                              */
/* Purpose:  This function determines the effect of the recall               */
/*           function on the RF level.                                       */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setRFLevelReset (ViSession instrSession,
                                         ViInt32 resetState)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    
    if (rssmr_invalidViInt32Range (resetState, 0, 1))    
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:POW:RCL %s\n", freqResetArr[resetState]));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set Fixed Power Mode                                            */
/* Purpose:  This function specifies the power operating mode where the      */
/*           output level is specified by means of functions under RF Level  */
/*           group of functions.                                             */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setFixedPowerMode (ViSession instrSession)
{
    ViStatus rssmr_status = VI_SUCCESS;

    CHECKERR (viPrintf (instrSession, ":SOUR:POW:MODE FIX\n"));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Configure RF Level Sweep.                                       */
/* Purpose:  This function sets the RF level, offset and limit.              */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_configRFLevelSweep (ViSession instrSession,
                                            ViInt32 mode, ViReal64 startLevel,
                                            ViReal64 stopLevel, ViReal64 step,
                                            ViReal64 dwellTime)
{
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;
    ViReal64    powMin = -130.0,
                powMax = 25.0;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B4);

    /* Query actual MIN and MAX values */
    CHECKERR (viQueryf (instrSession, ":SOUR:POW? MIN;POW? MAX\n", "%Le;%Le",
        &powMin, &powMax));

    if (rssmr_invalidViInt32Range (mode, 0, 2))  
       return VI_ERROR_PARAMETER2;
    if (rssmr_invalidViReal64Range (startLevel, powMin, powMax))
       return VI_ERROR_PARAMETER3;
    if (rssmr_invalidViReal64Range (stopLevel, powMin, powMax))
       return VI_ERROR_PARAMETER4;
    if (rssmr_invalidViReal64Range (step, 0.0, 160.0))   
       return VI_ERROR_PARAMETER5;
    if (rssmr_invalidViReal64Range (dwellTime, 0.01, 5.0))   
       return VI_ERROR_PARAMETER6;

    CHECKERR (viPrintf (instrSession, ":SOUR:SWE:POW:MODE %s"
        ";STEP %Lf;DWEL %Lfs;:SOUR:POW:STAR %Lf;STOP %Lf\n",
        SweepModeArr[mode], step, dwellTime, startLevel, stopLevel));
    CHECKERR (rssmr_checkStatus (instrSession));

    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set RF Level Sweep Mode.                                        */
/* Purpose:  This function specifies the sweep mode.                         */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setRFLevelSweepMode (ViSession instrSession,
                                             ViInt32 mode)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B4);
    
    if (rssmr_invalidViInt32Range (mode, 0, 2))  
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:SWE:POW:MODE %s\n", SweepModeArr[mode]));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set RF Start Level.                                             */
/* Purpose:  This function sets the staring value for a level sweep.         */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setRFStartLevel (ViSession instrSession, ViReal64 level)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B4);

    CHECKERR (viPrintf (instrSession, ":SOUR:POW:STAR %Lf\n", level));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set RF Stop Level.                                              */
/* Purpose:  This function sets the final value for a level sweep.           */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setRFStopFreqLevel (ViSession instrSession,
                                            ViReal64 level)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B4);

    CHECKERR (viPrintf (instrSession, ":SOUR:POW:STOP %Lf\n", level));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set RF Level Increment                                          */
/* Purpose:  This function defines the step width for the level setting.     */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setRFLevelIncrement (ViSession instrSession,
                                            ViReal64 increment)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    
    if (rssmr_invalidViReal64Range (increment, 0.1, 10.0))   
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:POW:STEP:INCR %Lf\n", increment));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set RF Level Sweep Step.                                        */
/* Purpose:  This function sets the sweep step.                              */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setRFLevelSweepStep (ViSession instrSession,
                                             ViReal64 step)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B4);
    
    if (rssmr_invalidViReal64Range (step, 0.0, 160.0))   
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:SWE:POW:STEP %Lf\n", step));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set RF Level Sweep Dweel Time.                                  */
/* Purpose:  This function sets the dwell time per level step.               */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setRFLevelSweepDwellTime (ViSession instrSession,
                                                  ViReal64 time)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B4);
    
    if (rssmr_invalidViReal64Range (time, 0.01, 5.0))    
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:SWE:POW:DWEL %Lfs\n", time));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set RF Level Sweep Manual.                                      */
/* Purpose:  This function sets the RF sweep level in Manual mode.           */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setRFLevelSweepManual (ViSession instrSession,
                                               ViReal64 level)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B4);

    CHECKERR (viPrintf (instrSession, ":SOUR:POW:MAN %Lf\n", level));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set RF Level Sweep Spacing                                      */
/* Purpose:  This function defines that the sweep steps have logarithmic     */
/*           spacings.                                                       */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setRFLevelSweepSpacing (ViSession instrSession,
                                                ViInt32 spacing)
{
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B4);

    if (rssmr_invalidViInt32Range (spacing, 1, 1))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:SWE:POW:SPAC %s\n", "LOG"));
    CHECKERR (rssmr_checkStatus (instrSession));

    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set Sweep Power Mode                                            */
/* Purpose:  This function specifies the power operating mode where the      */
/*           instrument operates in the SWEep mode. The level is specified by*/
/*           means of functions under RF Level Sweep group of functions.     */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setSweepPowerMode (ViSession instrSession)
{
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B4);
    
    CHECKERR (viPrintf (instrSession, ":SOUR:POW:MODE SWE\n"));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set RF Level Unit                                               */
/* Purpose:  This function selects RF level unit.                            */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setRfLevelUnit (ViSession instrSession,
                                             ViInt32 levelUnit)
{                               
    ViStatus rssmr_status = VI_SUCCESS;

    if (rssmr_invalidViInt32Range (levelUnit, 0, 2))  
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:POW:UNIT %s\n", levelUnitArr[levelUnit]));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Configure RF Level Correction                                   */
/* Purpose:  This function sets the correction state, list name, frequency   */
/*           and power points.                                               */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_ConfigureRFLevelCorr (ViSession instrSession,
                                              ViBoolean state,
                                              ViString correctionListName,
                                              ViReal64 _VI_FAR frequencyPoints[],
                                              ViReal64 _VI_FAR powerPoints[],
                                              ViInt32 numberOfElements)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;    

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr))
    
    if (rssmr_invalidViBooleanRange (state))     
       return VI_ERROR_PARAMETER2;

    if (state) {
        if (rssmr_invalidViString (correctionListName, 7))   
           return VI_ERROR_PARAMETER3;
        if (!frequencyPoints)    
           return VI_ERROR_PARAMETER4;
        if (!powerPoints)    
           return VI_ERROR_PARAMETER5;

        /* VI_FALSE ... Firmware version 1.07 */
        /* VI_TRUE ...  Firmware version 1.08 or higher */
    
        if ((instrPtr -> firmware) == VI_TRUE) {
            if (numberOfElements > 0)   {
                CHECKERR (viPrintf (instrSession, ":SOUR:CORR:STAT ON;CSET:SEL \"%s\""
                    ";DATA:FREQ %,*Lf;:SOUR:CORR:CSET:DATA:POW %,*Lf\n", correctionListName,
                    numberOfElements, frequencyPoints,
                    numberOfElements, powerPoints));
            }
            else {
                CHECKERR (viPrintf (instrSession, ":SOUR:CORR:STAT ON;CSET:SEL \"%s\"\n",
                    correctionListName));
            }
        }
        else {
            if (numberOfElements > 0)   {
                CHECKERR (viPrintf (instrSession, ":SOUR:CORR:STAT ON;CSET:SEL %s"
                    ";DATA:FREQ %,*Lf;:SOUR:CORR:CSET:DATA:POW %,*Lf\n", correctionListName,
                    numberOfElements, frequencyPoints,
                    numberOfElements, powerPoints));
            }
            else {
                CHECKERR (viPrintf (instrSession, ":SOUR:CORR:STAT ON;CSET:SEL %s\n",
                    correctionListName));
            }
        }
    }
    else CHECKERR (viPrintf (instrSession, ":SOUR:CORR:STAT OFF\n"));

    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set RF Level Correction State                                   */
/* Purpose:  This function sets correction table on or off.                  */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setRFLevelCorrState (ViSession instrSession,
                                             ViBoolean state)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    
    if (rssmr_invalidViBooleanRange (state))     
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:CORR:STAT %s\n", state ? "ON" : "OFF"));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Select / Create Correction List                                 */
/* Purpose:  This function select or creates correction list.                */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_selCorrList (ViSession instrSession,
                                     ViString correctionListName)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;    

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr))

    if (rssmr_invalidViString (correctionListName, 7))   
       return VI_ERROR_PARAMETER2;

    /* VI_FALSE ... Firmware version 1.07 */
    /* VI_TRUE ...  Firmware version 1.08 or higher */
    
    if ((instrPtr -> firmware) == VI_TRUE) {
        CHECKERR (viPrintf (instrSession, ":SOUR:CORR:CSET:SEL \"%s\"\n",
            correctionListName));
    }
    else {
        CHECKERR (viPrintf (instrSession, ":SOUR:CORR:CSET:SEL %s\n",
            correctionListName));
    }

    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Delete Correction List                                          */
/* Purpose:  This function deletes correction list.                          */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_deleteCorrList (ViSession instrSession,
                                        ViString correctionListName)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;    

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr))
    
    if (rssmr_invalidViString (correctionListName, 7))   
       return VI_ERROR_PARAMETER2;

    /* VI_FALSE ... Firmware version 1.07 */
    /* VI_TRUE ...  Firmware version 1.08 or higher */

    if ((instrPtr -> firmware) == VI_TRUE) {
        CHECKERR (viPrintf (instrSession, ":SOUR:CORR:CSET:DEL \"%s\"\n",
            correctionListName));
    }
    else {
        CHECKERR (viPrintf (instrSession, ":SOUR:CORR:CSET:DEL %s\n",
            correctionListName));
    }
    
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Delete All Correction Lists                                     */
/* Purpose:  This function deletes all correction lists.                     */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_deleteAllCorrLists (ViSession instrSession)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    
    CHECKERR (viPrintf (instrSession, ":SOUR:CORR:CSET:DEL:ALL\n"));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Get List Of Correction Lists                                    */
/* Purpose:  This function returns list of correction lists.                 */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_getListOfCorrLists (ViSession instrSession,
                                            ViChar _VI_FAR listOfCorrectionLists[])
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    
    if (!listOfCorrectionLists)  
       return VI_ERROR_PARAMETER2;

    CHECKERR (viQueryf (instrSession, ":SOUR:CORR:CSET:CAT?\n", "%[^\r\n]",
        listOfCorrectionLists));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set Frequency To Correction List                                */
/* Purpose:  This function sets the frequency array to correction list.      */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setFreqToCorrList (ViSession instrSession,
                                           ViReal64 _VI_FAR frequency[],
                                           ViInt32 size)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    
    if (!frequency)
       return VI_ERROR_PARAMETER2;
    if ( size < 1 )
       return VI_ERROR_PARAMETER3;

    CHECKERR (viPrintf (instrSession, ":SOUR:CORR:CSET:DATA:FREQ %,*Lf\n",
        size, frequency));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set Power To Correction List                                    */
/* Purpose:  This function sets the power array to correction list.          */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setPowerToCorrList (ViSession instrSession,
                                            ViReal64 _VI_FAR power[],
                                            ViInt32 size)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    
    if (!power)
       return VI_ERROR_PARAMETER2;
    if ( size < 1 )
       return VI_ERROR_PARAMETER3;

    CHECKERR (viPrintf (instrSession, ":SOUR:CORR:CSET:DATA:POW %,*Lf\n",
        size, power));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Get Level Correction Points                                     */
/* Purpose:  This function returns the number of correction list elements.   */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_getLevelCorrPoints (ViSession instrSession,
                                            ViPInt32 points)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    
    if (!points)
       return VI_ERROR_PARAMETER2;

    CHECKERR (viQueryf (instrSession, ":SOUR:CORR:CSET:DATA:POW:POIN?\n", "%ld",
        points));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Get Level Frequency Points                                      */
/* Purpose:  This function returns the number of correction list elements.   */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_getFreqCorrPoints (ViSession instrSession,
                                            ViPInt32 points)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    
    if (!points)
       return VI_ERROR_PARAMETER2;

    CHECKERR (viQueryf (instrSession, ":SOUR:CORR:CSET:DATA:FREQ:POIN?\n", "%ld",
        points));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Get Free Space                                                  */
/* Purpose:  This function queries the free space in the Ucor table.         */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_getFreeCorrTables (ViSession instrSession,
                                           ViPInt32 freeSpace)
{
    ViStatus rssmr_status = VI_SUCCESS;
    
    if (!freeSpace)
       return VI_ERROR_PARAMETER2;

    CHECKERR (viQueryf (instrSession, ":SOUR:CORR:CSET:FREE?\n", "%ld", freeSpace));
    CHECKERR (rssmr_checkStatus (instrSession));

    return rssmr_status;
}

/*===========================================================================*/
/* Function: Configure RF Output                                             */
/* Purpose:  This function sets the attenuation mode, attenuation and        */
/*           power on state.                                                 */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_configRFOutput (ViSession instrSession,
                                        ViBoolean state,
                                        ViInt32 attenuationMode,
                                        ViReal64 attenuation)
{
    ViStatus rssmr_status = VI_SUCCESS;

    if (rssmr_invalidViBooleanRange (state))
       return VI_ERROR_PARAMETER2;

    if (state) {
        if (rssmr_invalidViInt32Range (attenuationMode, 0, 1))
           return VI_ERROR_PARAMETER3;
        if (rssmr_invalidViReal64Range (attenuation, 0.0, 110.0))
           return VI_ERROR_PARAMETER4;
    
        CHECKERR (viPrintf (instrSession, ":OUTP1:STAT ON;AMOD %s;ATT %Lf\n",
        RFAttModeArr[attenuationMode], attenuation));
    }
    else CHECKERR (viPrintf (instrSession, ":OUTP1:STAT OFF\n"));

    CHECKERR (rssmr_checkStatus (instrSession));

    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set RF Output State                                             */
/* Purpose:  This function sets RF output on or off.                         */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setRFOutputState (ViSession instrSession,
                                          ViBoolean state)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    
    if (rssmr_invalidViBooleanRange (state))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":OUTP1:STAT %s\n", state ? "ON" : "OFF"));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set RF Attenuation Mode                                         */
/* Purpose:  This function sets RF attenuation mode.                         */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setRFAttMode (ViSession instrSession,
                                      ViInt32 attenuationMode)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    
    if (rssmr_invalidViInt32Range (attenuationMode, 0, 1))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":OUTP1:AMOD %s\n", RFAttModeArr[attenuationMode]));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set RF Attenuation                                              */
/* Purpose:  This function sets attenuation value of the RF output.          */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setRFAttenuation (ViSession instrSession,
                                          ViReal64 attenuation)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    
    if (rssmr_invalidViReal64Range (attenuation, 0.0, 110.0))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":OUTP1:ATT %Lf\n", attenuation));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Get RF Output Impedance                                         */
/* Purpose:  This function returns RF output impedance.                      */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_getRFOutputImpedance (ViSession instrSession,
                                              ViPReal64 impedance)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    
    if (!impedance)
       return VI_ERROR_PARAMETER2;

    CHECKERR (viQueryf (instrSession, ":OUTP1:IMP?\n", "%Lf", impedance));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set RF Output Power On State                                    */
/* Purpose:  This function sets the state of RF output after reset.          */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setRFOutPowOnState (ViSession instrSession,
                                            ViInt32 state)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    
    if (rssmr_invalidViInt32Range (state,0,1))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":OUTP1:PON %s\n", RFOutPonSArr[state]));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Configure AM Modulation                                         */
/* Purpose:  This function sets the modulation depth, frequency and          */
/*           source.                                                         */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_configAMModulation (ViSession instrSession, ViInt32 source,
                                            ViReal64 depth, ViReal64 frequency)
{
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B5);

    if (rssmr_invalidViInt32Range (source,0,2))
       return VI_ERROR_PARAMETER2;
    if (rssmr_invalidViReal64Range (depth, 0.0, 100.0))
       return VI_ERROR_PARAMETER3;
    if (rssmr_invalidViReal64Range (frequency, 0.1, 10.0e6))
       return VI_ERROR_PARAMETER4;
    
    CHECKERR (viPrintf (instrSession, ":SOUR:AM:STAT ON;SOUR %s;DEPT %Lf"
        ";INT:FREQ %Lf\n", AMSourceArr[source], depth, frequency));
    CHECKERR (rssmr_checkStatus (instrSession));

    return rssmr_status;
}

/*===========================================================================*/
/* Function: Configure FM Modulation                                         */
/* Purpose:  This function sets the modulation deviation, frequency and      */
/*           source.                                                         */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_configFMModulation (ViSession instrSession, ViInt32 source,
                                            ViReal64 deviation, ViReal64 frequency)
{
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B5);

    if (rssmr_invalidViInt32Range (source,0,2))
       return VI_ERROR_PARAMETER2;
    if (rssmr_invalidViReal64Range (deviation, 0.0, 40.0e6))
       return VI_ERROR_PARAMETER3;
    if (rssmr_invalidViReal64Range (frequency, 0.1, 10.0e6))
       return VI_ERROR_PARAMETER4;

    CHECKERR (viPrintf (instrSession, ":SOUR:FM:STAT ON;DEV %Lf;SOUR %s"
        ";INT:FREQ %Lf\n", deviation, AMSourceArr[source], frequency));
    CHECKERR (rssmr_checkStatus (instrSession));

    return rssmr_status;
}

/*===========================================================================*/
/* Function: Configure PM Modulation                                         */
/* Purpose:  This function sets the modulation polarity and source           */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_configPulseModulation (ViSession instrSession,
                                               ViInt32 source, ViInt32 polarity)
{
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B14);

    if (rssmr_invalidViInt32Range (source, 0, 1))
       return VI_ERROR_PARAMETER2;
    if (rssmr_invalidViInt32Range (polarity, 0, 1))
       return VI_ERROR_PARAMETER3;
    
    CHECKERR (viPrintf (instrSession, ":SOUR:PULM:STAT ON;POL %s;SOUR %s\n",
        PolarityArr[polarity], PMSourceArr[source]));
    CHECKERR (rssmr_checkStatus (instrSession));

    return rssmr_status;
}

/*===========================================================================*/
/* Function: Configure DM Modulation                                         */
/* Purpose:  This function sets the modulation type.                         */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_configDigitalModulation (ViSession instrSession,
                                                 ViInt32 DMType,
                                                 ViReal64 FSKDeviation,
                                                 ViInt32 FSKPolarity,
                                                 ViReal64 ASKDepth,
                                                 ViInt32 ASKPolarity)
{
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B5);

    if (rssmr_invalidViInt32Range (DMType,0,1))
       return VI_ERROR_PARAMETER2;
    if (rssmr_invalidViReal64Range (FSKDeviation, 0.0, 40.0e6))
       return VI_ERROR_PARAMETER3;
    if (rssmr_invalidViInt32Range (FSKPolarity,0,1))
       return VI_ERROR_PARAMETER4;
    if (rssmr_invalidViReal64Range (ASKDepth, 0.0, 100.0))
       return VI_ERROR_PARAMETER5;
    if (rssmr_invalidViInt32Range (ASKPolarity,0,1))
       return VI_ERROR_PARAMETER6;

    CHECKERR (viPrintf (instrSession, ":SOUR:DM:STAT ON;TYPE %s;FSK:DEV %Lf"
        ";:SOUR:DM:FSK:POL %s;:SOUR:DM:ASK:DEPT %Lf;:SOUR:DM:ASK:POL %s\n",
        DMTypeArr[DMType], FSKDeviation, PolarityArr[FSKPolarity], ASKDepth,
        PolarityArr[ASKPolarity]));
    CHECKERR (rssmr_checkStatus (instrSession));

    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set AM State                                                    */
/* Purpose:  This function sets amplitude modulation on or off.              */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setAMState (ViSession instrSession, ViBoolean state)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B5);
    
    if (rssmr_invalidViBooleanRange (state))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:AM:STAT %s\n", state ? "ON" : "OFF"));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set AM Depth                                                    */
/* Purpose:  This function sets amplitude modulation depth.                  */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setAMDepth (ViSession instrSession, ViReal64 depth)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B5);
    
    if (rssmr_invalidViReal64Range (depth, 0.0, 100.0))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:AM:DEPT %Lf\n", depth));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set AM Frequency                                                */
/* Purpose:  This function sets the modulation frequency.                    */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setAMFreq (ViSession instrSession, ViReal64 frequency)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B5);
    
    if (rssmr_invalidViReal64Range (frequency, 0.1, 10.0e6))     
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:AM:INT:FREQ %Lf\n", frequency));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set AM Source                                                   */
/* Purpose:  This function selects the modulation source        .            */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setAMSource (ViSession instrSession, ViInt32 source)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B5);
    
    if (rssmr_invalidViInt32Range (source, 0, 2))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:AM:SOUR %s\n", AMSourceArr[source]));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set AM External Coupling                                        */
/* Purpose:  This function selects the type of coupling for ext. source      */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setAMExtCoupling (ViSession instrSession,
                                          ViInt32 input, ViInt32 coupling)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B5);
    
    if (rssmr_invalidViInt32Range (input, 1, 2))
       return VI_ERROR_PARAMETER2;
    if (rssmr_invalidViInt32Range (coupling, 0, 1))
       return VI_ERROR_PARAMETER3;

    CHECKERR (viPrintf (instrSession, ":SOUR:AM:EXT%d:COUP %s\n", input,
        AMExtCoupArr[coupling]));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set AM External Impedance                                       */
/* Purpose:  This function defines the input impedance of the external input.*/
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setAMExtImp (ViSession instrSession, ViInt32 input,
                                     ViInt32 impedance)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B5);
    
    if (rssmr_invalidViInt32Range (input, 1, 2))
       return VI_ERROR_PARAMETER2;
    if (rssmr_invalidViInt32Range (impedance, 0, 1))
       return VI_ERROR_PARAMETER3;

    CHECKERR (viPrintf (instrSession, ":SOUR:AM:EXT%d:IMP %s\n", input,
        AMExtImpArr[impedance]));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set AM Scan State                                               */
/* Purpose:  This function switches the logarithmic amplitude modulation     */
/*           on or off.                                                      */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setAMScanState (ViSession instrSession, ViBoolean state)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B5);
    
    if (rssmr_invalidViBooleanRange (state))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:AM:SCAN:STAT %s\n", state ? "ON" : "OFF"));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set AM Scan Sensitivity                                         */
/* Purpose:  This function sets the scan sensitivity.                        */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setAMScanSens (ViSession instrSession,
                                       ViReal64 sensitivity)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B5);
    
    if (rssmr_invalidViReal64Range (sensitivity, -0.1, 10.0))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:AM:SCAN:SENS %Lf\n", sensitivity));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set FM State                                                    */
/* Purpose:  This function sets frequency modulation on or off.              */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setFMState (ViSession instrSession, ViBoolean state)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B5);
 
    if (rssmr_invalidViBooleanRange (state))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:FM:STAT %s\n", state ? "ON" : "OFF"));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set FM Deviation.                                               */
/* Purpose:  This function sets the frequency variation.                     */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setFMDeviation (ViSession instrSession,
                                        ViReal64 deviation)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B5);
    
    if (rssmr_invalidViReal64Range (deviation, 0.0, 40.0e6))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:FM:DEV %Lf\n", deviation));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set FM Frequency                                                */
/* Purpose:  This function sets the frequency.                               */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setFMFreq (ViSession instrSession, ViReal64 frequency)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B5);
    
    if (rssmr_invalidViReal64Range (frequency, 0.1, 10.0e6))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:FM:INT:FREQ %Lf\n", frequency));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set FM Source                                                   */
/* Purpose:  This function selest the modulation source.                     */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setFMSource (ViSession instrSession, ViInt32 source)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B5);
    
    if (rssmr_invalidViInt32Range (source, 0, 2))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:FM:SOUR %s\n", AMSourceArr[source]));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set FM External Coupling                                        */
/* Purpose:  This function selects the type of coupling for ext. source      */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setFMExtCoup (ViSession instrSession, ViInt32 input,
                                      ViInt32 coupling)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B5);
    
    if (rssmr_invalidViInt32Range (input, 1, 2))
       return VI_ERROR_PARAMETER2;
    if (rssmr_invalidViInt32Range (coupling, 0, 1))
       return VI_ERROR_PARAMETER3;

    CHECKERR (viPrintf (instrSession, ":SOUR:FM:EXT%d:COUP %s\n", input,
        AMExtCoupArr[coupling]));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set FM External Impedance                                       */
/* Purpose:  This function defines the input impedance of the external input.*/
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setFMExtImp (ViSession instrSession, ViInt32 input,
                                     ViInt32 impedance)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B5);
    
    if (rssmr_invalidViInt32Range (input, 1, 2))
       return VI_ERROR_PARAMETER2;
    if (rssmr_invalidViInt32Range (impedance, 0, 1))
       return VI_ERROR_PARAMETER3;

    CHECKERR (viPrintf (instrSession, ":SOUR:FM:EXT%d:IMP %s\n", input,
        AMExtImpArr[impedance]));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set PM State                                                    */
/* Purpose:  This function sets pulse modulation on or off.                  */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setPMState (ViSession instrSession, ViBoolean state)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B14);
 
    if (rssmr_invalidViBooleanRange (state))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:PULM:STAT %s\n", state ? "ON" : "OFF"));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set PM Polarity                                                 */
/* Purpose:  This function sets the polarity between modulating and          */
/*           modulated signal.                                               */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setPMPolar (ViSession instrSession, ViInt32 polarity)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B14);
    
    if (rssmr_invalidViInt32Range (polarity, 0, 1))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:PULM:POL %s\n", PolarityArr[polarity]));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set PM Source                                                   */
/* Purpose:  This function select the source of the modulating signal        */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setPMSource (ViSession instrSession, ViInt32 source)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B14);

    if (rssmr_invalidViInt32Range (source, 0, 1))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:PULM:SOUR %s\n", PMSourceArr[source]));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set PM External Impedance                                       */
/* Purpose:  This function sets the impedance of the input socket for        */
/*           the external pulse generator.                                   */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setPMExtImp (ViSession instrSession, ViReal64 impedance)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B14);

    if (rssmr_invalidViReal64Range (impedance, 50.0, 10.0e3))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:PULM:EXT:IMP %Lf\n", impedance));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set DM State                                                    */
/* Purpose:  This function sets digital modulation on or off.                */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setDMState (ViSession instrSession, ViBoolean state)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B5);
 
    if (rssmr_invalidViBooleanRange (state))     
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:DM:STAT %s\n", state ? "ON" : "OFF"));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set DM Type                                                     */
/* Purpose:  This function selects the modulation type.                      */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setDMType (ViSession instrSession, ViInt32 type)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B5);
    
    if (rssmr_invalidViInt32Range (type, 0, 1))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:DM:TYPE %s\n", DMTypeArr[type]));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set FSK Deviation.                                              */
/* Purpose:  This function sets the frequency variation of the FSK.          */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setFSKDeviation (ViSession instrSession,
                                         ViReal64 deviation)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B5);
    
    if (rssmr_invalidViReal64Range (deviation, 0.0, 40.0e6))     
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:DM:FSK:DEV %Lf\n", deviation));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set FSK Polarity                                                */
/* Purpose:  This function sets the polarity of FSK modulation.              */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setFSKPolar (ViSession instrSession, ViInt32 polarity)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B5);
    
    if (rssmr_invalidViInt32Range (polarity, 0, 1))  
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:DM:FSK:POL %s\n", PolarityArr[polarity]));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set ASK Depth.                                                  */
/* Purpose:  This function sets the modulation depth of the ASK.             */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setASKDepth (ViSession instrSession, ViReal64 depth)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B5);
    
    if (rssmr_invalidViReal64Range (depth, 0.0, 100.0))  
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:DM:ASK:DEPT %Lf\n", depth));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set ASK Polarity                                                */
/* Purpose:  This function sets the polarity of ASK modulation.              */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setASKPolar (ViSession instrSession, ViInt32 polarity)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B5);
    
    if (rssmr_invalidViInt32Range (polarity, 0, 1))  
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:DM:ASK:POL %s\n", PolarityArr[polarity]));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set DM External Impedance                                       */
/* Purpose:  This function sets the imput impedance of the external          */
/*           DM-input.                                                       */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setDMExtImp (ViSession instrSession, ViInt32 impedance)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B5);

    if (rssmr_invalidViInt32Range (impedance, 0, 1))     
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:DM:EXT:IMP %s\n", AMExtImpArr[impedance]));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Configure ALC                                                   */
/* Purpose:  This function sets the level control on or off and selects      */
/*           its source and reference.                                       */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_configureALC (ViSession instrSession,
                ViBoolean levelControl, ViInt32 source, ViReal64 reference)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
 
    if (rssmr_invalidViBooleanRange (levelControl))  
       return VI_ERROR_PARAMETER2;
    if (rssmr_invalidViInt32Range (source, 0, 3))    
       return VI_ERROR_PARAMETER3;
    if (rssmr_invalidViReal64Range (reference, 0.0, 3.0))    
       return VI_ERROR_PARAMETER2;

    if (levelControl) {
        if (source < 2) {
            CHECKERR (viPrintf (instrSession, ":SOUR:POW:ALC:STAT ON;SOUR %s;REF %LfV\n",
                                              ALCSourArr[source], reference));
        }
        else {
            CHECKERR (viPrintf (instrSession, ":SOUR:POW:ALC:STAT ON;REF %LfV"
                                              ";SOUR PMET;SOUR:PMET %s\n",
                                              reference, ALCSourArr[source]));
        }
    }
    else CHECKERR (viPrintf (instrSession, ":SOUR:POW:ALC:STAT OFF\n"));

    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set ALC Reference                                               */
/* Purpose:  This function sets the reference voltage.                       */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setALCReference (ViSession instrSession,
                                         ViReal64 reference)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    
    if (rssmr_invalidViReal64Range (reference, 0.0, 3.0))    
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:POW:ALC:REF %LfV\n", reference));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: ALC Search                                                      */
/* Purpose:  This function returns ALC search results.                       */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_ALCSearch (ViSession instrSession,
                                   ViChar _VI_FAR searchResult[])
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    
    if (!searchResult)   
       return VI_ERROR_PARAMETER2;

    CHECKERR (viQueryf (instrSession, ":SOUR:POW:ALC:SEARCH?\n", "%s", searchResult));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set ALC Source                                                  */
/* Purpose:  This function sets the ALC source.                              */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setALCSource (ViSession instrSession, ViInt32 source)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    
    if (rssmr_invalidViInt32Range (source, 0, 3))    
       return VI_ERROR_PARAMETER2;

    if (source <= 1) {
        CHECKERR (viPrintf (instrSession, ":SOUR:POW:ALC:SOUR %s\n", ALCSourArr[source]));
    }
    else {
        CHECKERR (viPrintf (instrSession, ":SOUR:POW:ALC:SOUR PMET"
            ";:SOUR:POW:ALC:SOUR:PMET %s\n", ALCSourArr[source]));
    }

    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set ALC Slope                                                   */
/* Purpose:  This function selects the RF power slew rate for level settings */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setALCSlope (ViSession instrSession, ViInt32 slope)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    
    if (rssmr_invalidViInt32Range (slope, 0, 2))    
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:POW:ALC:SLOP %s\n", ALCslopeArr[slope]));

    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set ALC State                                                   */
/* Purpose:  This function sets the ALC on or off.                           */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setALCState (ViSession instrSession, ViBoolean state)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
 
    if (rssmr_invalidViBooleanRange (state))     
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:POW:ALC:STAT %s\n", state ? "ON" : "OFF"));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Configure LF Output                                             */
/* Purpose:  This function sets the output voltage, frequency, function and  */
/*           mode.                                                           */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_configLFOutput (ViSession instrSession, ViBoolean state,
                                        ViInt32 function, ViReal64 frequency,
                                        ViReal64 voltage)
{
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B5);

    if (rssmr_invalidViBooleanRange (state))     
       return VI_ERROR_PARAMETER2;

    if (state) {
        if (rssmr_invalidViInt32Range (function,0, 1))
           return VI_ERROR_PARAMETER3;
        if (rssmr_invalidViReal64Range (frequency, 0.1, 10.0e6))
           return VI_ERROR_PARAMETER4;
        if (rssmr_invalidViReal64Range (voltage, 0.0, 4.0))
           return VI_ERROR_PARAMETER5;

        CHECKERR (viPrintf (instrSession, ":SOUR2:FREQ:MODE CW;:OUTP2:STAT ON;"
            ":SOUR2:FUNC:SHAP %s;:OUTP2:VOLT %LfV;:SOUR2:FREQ:CW %Lf\n",
            LFFuncArr[function], voltage, frequency));
    }
    else CHECKERR (viPrintf (instrSession, ":OUTP2:STAT OFF\n"));

    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set LF Voltage                                                  */
/* Purpose:  This function sets the ALC on or off.                           */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setLFVoltage (ViSession instrSession, ViReal64 voltage)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B5);
    
    if (rssmr_invalidViReal64Range (voltage, 0.0, 4.0))  
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":OUTP2:VOLT %LfV\n", voltage));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set LF Output State                                             */
/* Purpose:  This function sets the LF output on or off.                     */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setLFOutputState (ViSession instrSession,
                                          ViBoolean state)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B5);
 
    if (rssmr_invalidViBooleanRange (state))     
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":OUTP2:STAT %s\n", state ? "ON" : "OFF"));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set LF CW Frequency                                             */
/* Purpose:  This function sets the LF frequency.                            */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setLFCWFreq (ViSession instrSession, ViReal64 frequency)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B5);
    
    if (rssmr_invalidViReal64Range (frequency, 0.1, 10.0e6))     
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR2:FREQ:CW %Lf\n", frequency));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set LF Frequency Mode                                           */
/* Purpose:  This function selects the operating mode of LF subsystem.       */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setLFFreqMode (ViSession instrSession, ViInt32 mode)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B5);

    if (rssmr_invalidViInt32Range (mode, 0, 1))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR2:FREQ:MODE %s\n", RFFreqModeArr[mode]));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set LF Function                                                 */
/* Purpose:  This function selects LF subsystem function            .        */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setLFFunction (ViSession instrSession, ViInt32 function)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B5);

    if (rssmr_invalidViInt32Range (function, 0, 1))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR2:FUNC:SHAP %s\n", LFFuncArr[function]));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Configure LF Sweep                                              */
/* Purpose:  This function sets the sweep mode, spacing, dwell time and step.*/
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_configLFSweep (ViSession instrSession, ViInt32 mode,
                                       ViReal64 startFrequency,
                                       ViReal64 stopFrequency, ViReal64 step,
                                       ViReal64 dwellTime)
{
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B5);

    if (rssmr_invalidViInt32Range (mode, 0, 2))
       return VI_ERROR_PARAMETER2;
    if (rssmr_invalidViReal64Range (startFrequency, 0.1, 10.0e6))    
       return VI_ERROR_PARAMETER3;
    if (rssmr_invalidViReal64Range (stopFrequency, 0.1, 10.0e6))     
       return VI_ERROR_PARAMETER4;
    if (rssmr_invalidViReal64Range (step, 0.0, 10.0e6))  
       return VI_ERROR_PARAMETER5;
    if (rssmr_invalidViReal64Range (dwellTime, 0.01, 5.0))   
       return VI_ERROR_PARAMETER6;

    CHECKERR (viPrintf (instrSession, ":SOUR2:FREQ:MODE SWE;:SOUR2:SWE:FREQ:MODE %s;"
        ":SOUR2:FREQ:STAR %Lf;STOP %Lf;:SOUR2:SWE:FREQ:DWELL %Lfs;STEP:LIN %Lf\n",
        SweepModeArr[mode], startFrequency, stopFrequency, dwellTime, step));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set LF Sweep Mode                                               */
/* Purpose:  This function specifies the run of the sweep.                   */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setLFSweepMode (ViSession instrSession, ViInt32 mode)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B5);
    
    if (rssmr_invalidViInt32Range (mode, 0, 2))  
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR2:SWE:FREQ:MODE %s\n", SweepModeArr[mode]));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set LF CW Start Frequency                                       */
/* Purpose:  This function sets the LF start frequency.                      */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setLFStartFreq (ViSession instrSession,
                                        ViReal64 frequency)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B5);
    
    if (rssmr_invalidViReal64Range (frequency, 0.1, 10.0e6))     
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR2:FREQ:STAR %Lf\n", frequency));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set LF CW Stop Frequency                                        */
/* Purpose:  This function sets the LF stop frequency.                       */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setLFStopFreq (ViSession instrSession,
                                       ViReal64 frequency)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B5);
    
    if (rssmr_invalidViReal64Range (frequency, 0.1, 10.0e6))     
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR2:FREQ:STOP %Lf\n", frequency));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set LF CW Frequency Manual                                      */
/* Purpose:  This function sets the LF frequency in manual mode.             */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setLFFreqMan (ViSession instrSession,
                                      ViReal64 frequency)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B5);
    
    if (rssmr_invalidViReal64Range (frequency, 0.1, 10.0e6))     
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR2:FREQ:MAN %Lf\n", frequency));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set LF Sweep Spacing                                            */
/* Purpose:  This function specifies the steps have linear or logarithmic    */
/*           spacing                                                         */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setLFSweepSpac (ViSession instrSession, ViInt32 spacing)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B5);
    
    if (rssmr_invalidViInt32Range (spacing, 0, 1))   
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR2:SWE:FREQ:SPAC %s\n",
        LFFreqSweepSpacArr[spacing]));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set LF Sweep Step                                               */
/* Purpose:  This function specifies the step width in linear or logarithmic */
/*           mode.                                                           */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setLFSweepStep (ViSession instrSession, ViInt32 mode,
                                        ViReal64 step)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B5);
    
    if (rssmr_invalidViInt32Range (mode, 0, 1))  
       return VI_ERROR_PARAMETER2;

    if (mode == 0) {
        if (rssmr_invalidViReal64Range (step, 0.0, 10.0e6))  
           return VI_ERROR_PARAMETER2;
    }           
    else {
        if (rssmr_invalidViReal64Range (step, 0.01, 100.0))  
           return VI_ERROR_PARAMETER2;
    }           
    
    CHECKERR (viPrintf (instrSession, ":SOUR2:SWE:FREQ:STEP:%s %Lf\n",
        LFFreqSweepSpacArr[mode], step));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set LF Sweep Blank Time                                         */
/* Purpose:  This function defines the blank time of the sweep.              */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setLFSweepBlankTime (ViSession instrSession,
                                             ViInt32 blankTime)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B5);
    
    if (rssmr_invalidViInt32Range (blankTime, 0, 1))     
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR2:SWE:BTIM %s\n",
        sweepBTimeArr[blankTime]));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set LF Sweep Dweel Time.                                        */
/* Purpose:  This function sets the dwell time per frequency step.           */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setLFSweepDwellTime (ViSession instrSession,
                                             ViReal64 time)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B5);
    
    if (rssmr_invalidViReal64Range (time, 0.01, 5.0))    
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR2:SWE:FREQ:DWEL %Lfs\n", time));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set LF Sweep Time.                                              */
/* Purpose:  This function sets the sweep time from the start frequency      */
/*           to the stop frequency                                           */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setLFSweepTime (ViSession instrSession,
                                             ViReal64 time)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B5);
    
    CHECKERR (viPrintf (instrSession, ":SOUR:SWE:FREQ:TIME %Lfs\n", time));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Configure LF Marker                                             */
/* Purpose:  This function sets the marker state, frequency and polarity.    */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_configureLFMarker (ViSession instrSession,
                                           ViInt32 marker, ViBoolean state,
                                           ViReal64 frequency, ViInt32 polarity)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B5);
 
    if (rssmr_invalidViInt32Range (marker, 1, 3))
       return VI_ERROR_PARAMETER2;
    if (rssmr_invalidViBooleanRange (state))
       return VI_ERROR_PARAMETER3;

    if (state) {
        if (rssmr_invalidViReal64Range (frequency, 0.1, 10.0e6))
           return VI_ERROR_PARAMETER4;
        if (rssmr_invalidViInt32Range (polarity, 0, 1))
           return VI_ERROR_PARAMETER5;

        CHECKERR (viPrintf (instrSession, ":SOUR2:MARK%d ON;:SOUR2:MARK%d:FREQ %Lf"
            ";:SOUR2:MARK%d:POL %s\n", marker, marker, frequency, marker,
            PolarityArr[polarity]));
    }
    else CHECKERR (viPrintf (instrSession, ":SOUR2:MARK%d:FSW:STAT OFF\n", marker)); 

    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set LF Marker State                                             */
/* Purpose:  This function sets the marker on or off.                        */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setLFMarkState (ViSession instrSession, ViInt32 marker,
                                        ViBoolean state)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B5);
 
    if (rssmr_invalidViInt32Range (marker, 1, 3))
       return VI_ERROR_PARAMETER2;
    if (rssmr_invalidViBooleanRange (state))     
       return VI_ERROR_PARAMETER3;

    CHECKERR (viPrintf (instrSession, ":SOUR2:MARK%d:FSW:STAT %s\n",
        marker, state ? "ON" : "OFF"));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set LF Marker Frequency                                         */
/* Purpose:  This function sets the marker frequency.                        */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setLFMarkFreq (ViSession instrSession, ViInt32 marker,
                                       ViReal64 frequency)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B5);
    
    if (rssmr_invalidViInt32Range (marker, 1, 3))
       return VI_ERROR_PARAMETER2;
    if (rssmr_invalidViReal64Range (frequency, 0.1, 10.0e6))     
       return VI_ERROR_PARAMETER3;

    CHECKERR (viPrintf (instrSession, ":SOUR2:MARK%d:FSW:FREQ %Lf\n", marker,
        frequency));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set LF Marker Polarity                                          */
/* Purpose:  This function sets the polarity of marker signal.               */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setLFMarkPolar (ViSession instrSession, ViInt32 marker,
                                        ViInt32 polarity)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B5);

    if (rssmr_invalidViInt32Range (polarity, 0, 1))
       return VI_ERROR_PARAMETER3;

    CHECKERR (viPrintf (instrSession, ":SOUR2:MARK:POL %s\n", PolarityArr[polarity]));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Configure Pulse Output.                                         */
/* Purpose:  This function sets the pulse source, polarity, pulse period,    */
/*           width, delay and V/GHz scale                                    */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_configPulseOutput (ViSession instrSession,
                                           ViInt32 polarity, ViReal64 pulsePeriod,
                                           ViReal64 pulseWidth, ViReal64 pulseDelay)
{
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B14);

    if (rssmr_invalidViInt32Range (polarity, 0, 1))
       return VI_ERROR_PARAMETER2;
    if (rssmr_invalidViReal64Range (pulsePeriod, 100.0e-9, 85.0))
       return VI_ERROR_PARAMETER3;
    if (rssmr_invalidViReal64Range (pulseWidth, 20.0e-9, 1.3))
       return VI_ERROR_PARAMETER4;
    if (rssmr_invalidViReal64Range (pulseDelay, 20.0e-9, 1.3))
       return VI_ERROR_PARAMETER5;

    CHECKERR (viPrintf (instrSession, ":OUTP3:SOUR PULS;POL:PULS %s"
        ";:SOUR:PULS:PER %Les;WIDT %Les;DEL %Les\n", PolarityArr[polarity],
        pulsePeriod, pulseWidth, pulseDelay));
    CHECKERR (rssmr_checkStatus (instrSession));

    return rssmr_status;
}

/*===========================================================================*/
/* Function: Configure Video Output.                                         */
/* Purpose:  This function sets the video scale and sets video output on.    */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_configVideoOutput (ViSession instrSession,
                                           ViInt32 VGHzScale)
{
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B14);

    if (rssmr_invalidViInt32Range (VGHzScale, 0, 1))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":OUTP3:SOUR VID;:OUTP3:SCAL %s\n",
        VGHzScale ? "1" : "0.5"));
    CHECKERR (rssmr_checkStatus (instrSession));

    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set Pulse Source                                                */
/* Purpose:  This function selects the pulse generator or video output.      */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setPulseSource (ViSession instrSession, ViInt32 source)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B14);
    
    if (rssmr_invalidViInt32Range (source, 0, 2))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":OUTP3:SOUR %s\n", pulseOutSourceArr[source]));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set Pulse Polarity                                              */
/* Purpose:  This function selects the polarity of the signal.               */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setPulsePolarity (ViSession instrSession,
                                          ViInt32 polarity)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B14);
    
    if (rssmr_invalidViInt32Range (polarity,0,1))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":OUTP3:POL:PULS %s\n", PolarityArr[polarity]));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set Blank Signal Polarity                                       */
/* Purpose:  This function selects the polarity of the blank signal          */
/*           at the RF output.                                               */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setBlankSignalPolarity (ViSession instrSession,
                                          ViInt32 polarity)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B14);
    
    if (rssmr_invalidViInt32Range (polarity, 0, 1))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":OUTP1:BLAN:POL %s\n", PolarityArr[polarity]));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}
    
/*===========================================================================*/
/* Function: Set Pulse Period                                                */
/* Purpose:  This function sets the pulse period.                            */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setPulsePeriod (ViSession instrSession, ViReal64 period)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B14);
    
    if (rssmr_invalidViReal64Range (period, 100.0e-9, 85.0))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:PULS:PER %Les\n", period));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set RF Blanking Behavior Sync                                   */
/* Purpose:  This function switches RF blanking behavior synchronization on  */
/*           or off.                                                         */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setRfBlankingBehaviorSync (ViSession instrSession,
                                             ViBoolean state)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B14);
 
    if (rssmr_invalidViBooleanRange (state))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":OUTP1:RFBL:SYNC:STAT %s\n",
        state ? "ON" : "OFF"));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set RF Blanking Behavior                                        */
/* Purpose:  The command selects the RF blanking behavior for sweeps with    */
/*           steps under 10 MHz.                                             */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setRfBlankingBehavior (ViSession instrSession,
                                          ViInt32 RFBlankingBehavior)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B14);
    
    if (rssmr_invalidViInt32Range (RFBlankingBehavior, 0, 2))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":OUTP1:RFBL %s\n", RfBlankingArr[RFBlankingBehavior]));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}
    
/*===========================================================================*/
/* Function: Set V/GHz Output Scale                                          */
/* Purpose:  This function defines the signal scale at V/GHz output.         */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setVGHzOutputScale (ViSession instrSession,
                                            ViInt32 SignalScale)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B14);
    
    if (rssmr_invalidViInt32Range (SignalScale, 0, 1))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":OUTP3:SCAL %s\n", SignalScale ? "1" : "0.5"));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set Pulse Width                                                 */
/* Purpose:  This function sets the pulse width.                             */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setPulseWidth (ViSession instrSession, ViReal64 width)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B14);
    
    if (rssmr_invalidViReal64Range (width, 20.0e-9, 1.3))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:PULS:WIDT %Les\n", width));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set Pulse Delay                                                 */
/* Purpose:  This function sets the pulse delay.                             */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setPulseDelay (ViSession instrSession, ViReal64 delay)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B14);
    
    if (rssmr_invalidViReal64Range (delay, 20.0e-9, 1.3))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:PULS:DEL %Les\n", delay));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set Double Pulse State                                          */
/* Purpose:  This function switches the second pulse on or off.              */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setDoublePulseState (ViSession instrSession,
                                             ViBoolean state)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B14);
 
    if (rssmr_invalidViBooleanRange (state))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:PULS:DOUB:STAT %s\n",
        state ? "ON" : "OFF"));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set Double Pulse Delay                                          */
/* Purpose:  This function sets the double pulse delay.                      */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setDoublePulseDelay (ViSession instrSession,
                                             ViReal64 delay)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B14);
    
    if (rssmr_invalidViReal64Range (delay, 60.0e-9, 1.3))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:PULS:DOUB:DEL %Les\n", delay));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set IF Input State                                              */
/* Purpose:  This function switches the IF input on or off.                  */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setIFInputState (ViSession instrSession,
                                         ViBoolean state)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B23 | RSSMR_OPT_B24);
 
    if (rssmr_invalidViBooleanRange (state))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":INP:IF:STAT %s\n", state ? "ON" : "OFF"));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*============================================================================*/
/* Function: Configure Ref Oscillator State                                   */
/* Purpose:  This function sets ref. oscilator state, source and adjust value */
/*============================================================================*/
ViStatus _VI_FUNC rssmr_configureRefOscilator (ViSession instrSession,
                                               ViBoolean state, ViInt32 source,
                                               ViInt32 adjustValue)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B1);
 
    if (rssmr_invalidViBooleanRange (state))
       return VI_ERROR_PARAMETER2;

    if (state) {
        if (rssmr_invalidViInt32Range (source,0,1))
            return VI_ERROR_PARAMETER3;
        if (rssmr_invalidViInt32Range (adjustValue, -2048, 2047))
            return VI_ERROR_PARAMETER2;
        CHECKERR (viPrintf (instrSession, ":SOUR:ROSC:INT:ADJ:STAT ON;VAL %ld"
            ";:SOUR:ROSC:SOUR %s\n", adjustValue, RFOscSourceArr[source]));
    }
    else CHECKERR (viPrintf (instrSession, ":SOUR:ROSC:INT:ADJ:STAT OFF\n"));

    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set RF Oscillator State                                         */
/* Purpose:  This function switches the  frequency adjustment on or off.     */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setRFOscState (ViSession instrSession, ViBoolean state)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B1);
 
    if (rssmr_invalidViBooleanRange (state))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:ROSC:INT:ADJ:STAT %s\n",
        state ? "ON" : "OFF"));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set RF Oscillator Source                                        */
/* Purpose:  This function selects the reference source.                     */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setRefOscSource (ViSession instrSession, ViInt32 source)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B1);
    
    if (rssmr_invalidViInt32Range (source, 0, 1))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:ROSC:SOUR %s\n", RFOscSourceArr[source]));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set RF Oscillator Adjust Value                                  */
/* Purpose:  This function sets the frequency correction value.              */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setRefOscAdjVal (ViSession instrSession, ViInt32 value)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B1);
    
    if (rssmr_invalidViInt32Range (value, -2048, 2047))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:ROSC:INT:ADJ:VAL %ld\n", value));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Configure List                                                  */
/* Purpose:  This function selects the list mode, list, list time and        */
/*           frequency and power points                                      */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_configureList (ViSession instrSession, ViInt32 mode,
                                       ViString listname, ViReal64 time,
                                       ViReal64 _VI_FAR frequencyPoints[],
                                       ViReal64 _VI_FAR powerPoints[],
                                       ViInt32 numberOfElements)
{
    ViStatus rssmr_status = VI_SUCCESS;
    
    if (rssmr_invalidViInt32Range (mode, 0, 1))
       return VI_ERROR_PARAMETER2;
    if (rssmr_invalidViString (listname, 7))
       return VI_ERROR_PARAMETER3;
    if (rssmr_invalidViReal64Range (time, 0.001, 1.0))
       return VI_ERROR_PARAMETER4;

    if (numberOfElements > 0) {
        CHECKERR (viPrintf (instrSession, ":SOUR:LIST:MODE %s;SEL \"%s\""
            ";DWEL %Lfs;FREQ %,*Lf;POW %,*Lf\n", ListModeArr[mode], listname,
            time, numberOfElements, frequencyPoints, numberOfElements, powerPoints));
    }
    else {
        CHECKERR (viPrintf (instrSession, ":SOUR:LIST:MODE %s;SEL \"%s\";DWEL %Lf\n",
                        ListModeArr[mode], listname, time));
    }

    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set List Mode                                                   */
/* Purpose:  This function selects the list mode.                            */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setListMode (ViSession instrSession, ViInt32 mode)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    
    if (rssmr_invalidViInt32Range (mode, 0, 1))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:LIST:MODE %s\n", ListModeArr[mode]));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Select / Create List                                            */
/* Purpose:  This function select or creates list.                           */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_selList (ViSession instrSession, ViString listname)
{                               
    ViStatus rssmr_status = VI_SUCCESS;

    if (rssmr_invalidViString (listname, 7))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:LIST:SEL \"%s\"\n", listname));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Delete List                                                     */
/* Purpose:  This function deletes list.                                     */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_deleteList (ViSession instrSession, ViString listname)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;    

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr))
    
    if (rssmr_invalidViString (listname, 7))
       return VI_ERROR_PARAMETER2;

    /* VI_FALSE ... Firmware version 1.07 */
    /* VI_TRUE ...  Firmware version 1.08 or higher */
    
    if ((instrPtr -> firmware) == VI_TRUE) {
        CHECKERR (viPrintf (instrSession, ":SOUR:LIST:DEL \"%s\"\n", listname));
    }
    else {
        CHECKERR (viPrintf (instrSession, ":SOUR:LIST:DEL %s\n", listname));
    }

    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Delete All Lists                                                */
/* Purpose:  This function deletes all lists.                                */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_deleteAllLists (ViSession instrSession)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    
    CHECKERR (viPrintf (instrSession, ":SOUR:LIST:DEL:ALL\n"));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Get List Of Lists                                               */
/* Purpose:  This function returns list of lists.                            */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_getListOfLists (ViSession instrSession,
                                        ViChar _VI_FAR listOfLists[])
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    
    if (!listOfLists)    
       return VI_ERROR_PARAMETER2;

    CHECKERR (viQueryf (instrSession, ":SOUR:LIST:CAT?\n", "%[^\r\n]", listOfLists));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set List Dwell Time                                             */
/* Purpose:  This function sets the time the instrument dwells.              */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setListDwellTime (ViSession instrSession, ViReal64 time)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    
    if (rssmr_invalidViReal64Range (time, 0.001, 1.0))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:LIST:DWEL %Lfs\n", time));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Get List Dwell Points                                           */
/* Purpose:  This function returns the length of dwell part of list.         */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_getListDwellPoints (ViSession instrSession,
                                            ViPInt32 points)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    
    if (!points)     
       return VI_ERROR_PARAMETER2;

    CHECKERR (viQueryf (instrSession, ":SOUR:LIST:DWEL:POIN?\n", "%ld", points));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Get Space Properties                                            */
/* Purpose:  This function returns free and ocupied space.                   */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_getSpaceProperties (ViSession instrSession,
                                            ViPInt32 freeSpace,
                                            ViPInt32 occupiedSpace)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    
    if (!freeSpace)
       return VI_ERROR_PARAMETER2;

    if (!occupiedSpace)
       return VI_ERROR_PARAMETER3;

    CHECKERR (viQueryf (instrSession, ":SOUR:LIST:FREE?\n", "%ld,%ld",
        freeSpace, occupiedSpace));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set Frequency To List                                           */
/* Purpose:  This function sets the frequency array to list.                 */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setFreqToList (ViSession instrSession,
                                       ViReal64 _VI_FAR frequency[], ViInt32 size)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    
    if (!frequency)
       return VI_ERROR_PARAMETER2;
    if ( size < 1 )
       return VI_ERROR_PARAMETER3;

    CHECKERR (viPrintf (instrSession, ":SOUR:LIST:FREQ %,*Lf\n", size, frequency));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Get Frequency Points                                            */
/* Purpose:  This function returns the number of frequency points.           */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_getFreqPoints (ViSession instrSession, ViPInt32 points)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    
    if (!points)
       return VI_ERROR_PARAMETER2;

    CHECKERR (viQueryf (instrSession, ":SOUR:LIST:FREQ:POIN?\n", "%ld", points));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set Power To List                                               */
/* Purpose:  This function sets the power array to list.                     */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setPowerToList (ViSession instrSession,
                                        ViReal64 _VI_FAR power[], ViInt32 size)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    
    if (!power)
       return VI_ERROR_PARAMETER2;
    if ( size < 1 )
       return VI_ERROR_PARAMETER3;

    CHECKERR (viPrintf (instrSession, ":SOUR:LIST:POW %,*Lf\n", size, power));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Get Power Points                                                */
/* Purpose:  This function returns the number of power points.               */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_getPowerPoints (ViSession instrSession, ViPInt32 points)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    
    if (!points)
       return VI_ERROR_PARAMETER2;

    CHECKERR (viQueryf (instrSession, ":SOUR:LIST:POW:POIN?\n", "%ld", points));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set List Power Mode                                             */
/* Purpose:  This function specifies the power operating mode where the      */
/*           instrument processes a list of frequency and level settings. The*/
/*           settings are effected in the List Subsystem. Calling this       */
/*           function automatically sets RF Frequency Mode to the List mode  */
/*           as well.                                                        */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setListPowerMode (ViSession instrSession)
{
    ViStatus rssmr_status = VI_SUCCESS;
    
    CHECKERR (viPrintf (instrSession, ":SOUR:POW:MODE LIST\n"));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set List Blanking Behavior                                      */
/* Purpose:  The function selects the blanking behavior for list mode.       */                                                 
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setListBlankingBehavior (ViSession instrSession,
                                             ViBoolean state)
{                               
    ViStatus rssmr_status = VI_SUCCESS;

    if (rssmr_invalidViBooleanRange (state))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":OUTP1:LIBL %s\n",
        state ? "NORM" : "OFF"));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set Marker Freq Sweep State                                     */
/* Purpose:  This function sets selected marker on or off.                   */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setMarkerFreqSweepState (ViSession instrSession,
                                                 ViInt32 markerNo, ViBoolean state)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B4);
 
    if (rssmr_invalidViInt32Range (markerNo, 1, 10))     
       return VI_ERROR_PARAMETER2;
    if (rssmr_invalidViBooleanRange (state))     
       return VI_ERROR_PARAMETER3;

    CHECKERR (viPrintf (instrSession, ":SOUR:MARK%ld:FSW:STAT %s\n", markerNo,
        state ? "ON" : "OFF"));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set Marker Freq Sweep Amplitude                                 */
/* Purpose:  This function sets selected marker amplitude on or off.         */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setMarkerFreqSweepAmp (ViSession instrSession,
                                               ViInt32 markerNo,
                                               ViBoolean amplitude)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B4);
 
    if (rssmr_invalidViInt32Range (markerNo, 1, 10))     
       return VI_ERROR_PARAMETER2;
    if (rssmr_invalidViBooleanRange (amplitude))     
       return VI_ERROR_PARAMETER3;

    CHECKERR (viPrintf (instrSession, ":SOUR:MARK%ld:FSW:AMPL %s\n", markerNo,
        amplitude ? "ON" : "OFF"));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set Marker Freq Sweep Frequency                                 */
/* Purpose:  This function sets marker sweep frequency.                      */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setMarkerFreqSweepFreq (ViSession instrSession,
                                                ViInt32 markerNo,
                                                ViReal64 frequency)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B4);

    if (rssmr_invalidViInt32Range (markerNo, 1, 10))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:MARK%ld:FSW:FREQ %Lf\n", markerNo,
        frequency));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set Marker Level Sweep State                                    */
/* Purpose:  This function sets selected marker on or off.                   */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setMarkerLevelSweepState (ViSession instrSession,
                                                  ViInt32 markerNo, ViBoolean state)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B4);
 
    if (rssmr_invalidViInt32Range (markerNo, 1, 3))
       return VI_ERROR_PARAMETER2;
    if (rssmr_invalidViBooleanRange (state))
       return VI_ERROR_PARAMETER3;

    CHECKERR (viPrintf (instrSession, ":SOUR:MARK%ld:PSW:STAT %s\n", markerNo,
        state ? "ON" : "OFF"));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set Marker Freq Sweep Power                                     */
/* Purpose:  This function sets marker sweep power.                          */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setMarkerLevelSweepPower (ViSession instrSession,
                                                  ViInt32 markerNo, ViReal64 power)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B4);

    if (rssmr_invalidViInt32Range (markerNo, 1, 3))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:MARK%ld:PSW:POW %Le\n", markerNo, power));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set Sweep Trigger Source                                        */
/* Purpose:  This function selects sweep trigger source.                     */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setSweepTrigSource (ViSession instrSession,
                                            ViInt32 generator, ViInt32 source)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    
    if (rssmr_invalidViInt32Range (generator, 1, 2))
       return VI_ERROR_PARAMETER2;
    if (rssmr_invalidViInt32Range (source, 0, 2))
       return VI_ERROR_PARAMETER3;

    CHECKERR (viPrintf (instrSession, ":TRIG%ld:SWE:SOUR %s\n", generator,
        TriggerSourceArr[source]));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set List Trigger Source                                         */
/* Purpose:  This function selects list trigger source.                      */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setListTrigSource (ViSession instrSession,
                                           ViInt32 source)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    
    if (rssmr_invalidViInt32Range (source, 0, 2))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":TRIG:LIST:SOUR %s\n",
        TriggerSourceArr[source]));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set Pulse Trigger Source                                        */
/* Purpose:  This function selects pulse trigger source.                     */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setPulseTrigSource (ViSession instrSession,
                                            ViInt32 source)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    
    if (rssmr_invalidViInt32Range (source, 0, 2))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":TRIG:PULS:SOUR %s\n",
        PulseTriggerSourceArr[source]));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set Pulse Trigger Slope                                         */
/* Purpose:  This function selects pulse trigger slope.                      */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setPulseTrigSlope (ViSession instrSession,
                                           ViInt32 slope)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    
    if (rssmr_invalidViInt32Range (slope, 0, 1))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":TRIG:PULS:SLOP %s\n",
        PulseTriggerSlopeArr[slope]));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set Pulse Trigger Level                                         */
/* Purpose:  This function specifies the trigger level.                      */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setPulseTrigLevel (ViSession instrSession,
                                           ViInt32 level)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    
    if (rssmr_invalidViInt32Range (level, 0, 2))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":TRIG:PULS:LEV %s\n",
        PulseTriggerLevelArr[level]));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set Pulse Trigger Polarity                                      */
/* Purpose:  This function defines the active level of the gate signal       */
/*           (Normal - Active level = HIGH, Inverted Active level = LOW).    */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setPulseTrigPolarity (ViSession instrSession,
                                     ViInt32 triggerPolarity)
{
    ViStatus rssmr_status = VI_SUCCESS;

    if (rssmr_invalidViInt32Range (triggerPolarity, 0, 1))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":TRIG:PULS:EGAT:POL %s\n",
        PolarityArr[triggerPolarity]));
    CHECKERR (rssmr_checkStatus (instrSession));

    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set Slope Trigger                                               */
/* Purpose:  This function selects slope trigger.                            */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setSlopeTrig (ViSession instrSession, ViInt32 slope)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    
    if (rssmr_invalidViInt32Range (slope, 0, 2))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":TRIG:SLOP %s\n", TriggerSlopeArr[slope]));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set Level Calibration State                                     */
/* Purpose:  This function sets the level calibration on or off.             */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_setLevCalState (ViSession instrSession,
                                          ViBoolean state)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
 
    if (rssmr_invalidViBooleanRange (state))     
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":CAL:LEV:STAT %s\n", state ? "ON" : "OFF"));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Get Ref Osc Calib Value                                         */
/* Purpose:  This function gets calibration value for reference oscillator.  */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_getRefOscCalVal (ViSession instrSession,
                                         ViPInt32 value)
{
    ViStatus rssmr_status = VI_SUCCESS;
    
    if (!value)  
       return VI_ERROR_PARAMETER2;

    CHECKERR (viQueryf (instrSession, ":CAL:ROSC?\n", "%ld", value));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Store Ref Osc Calib Value                                       */
/* Purpose:  This function stores calibration value for reference oscillator.*/
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_storeRefOscCalVal (ViSession instrSession)
{
    ViStatus rssmr_status = VI_SUCCESS;

    CHECKERR (viPrintf (instrSession, ":CAL:ROSC:STOR\n"));
    CHECKERR (rssmr_checkStatus (instrSession));

    return rssmr_status;
}

/*===========================================================================*/
/* Function: Send Trigger                                                    */
/* Purpose:  This function sends selected trigger.                           */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_SendTrigger (ViSession instrSession, ViInt32 generator,
                                     ViInt32 systemToTrigger)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    
    if (rssmr_invalidViInt32Range (generator, 1, 2))
       return VI_ERROR_PARAMETER2;
    if (rssmr_invalidViInt32Range (systemToTrigger, 0, 0))
       return VI_ERROR_PARAMETER3;
    
    if (systemToTrigger == 0) {
        CHECKERR (viPrintf (instrSession, ":TRIG%d:%s:IMM\n", generator,
            TriggerArr[systemToTrigger]));  
    }
    else {
        CHECKERR (viPrintf (instrSession, ":TRIG:%s:IMM\n",
            TriggerArr[systemToTrigger]));
    }
    
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Abort Trigger                                                   */
/* Purpose:  This function aborts selected trigger.                          */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_abortTrigger (ViSession instrSession,
                                      ViInt32 systemToAbort)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    
    if (rssmr_invalidViInt32Range (systemToAbort, 0, 2))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":ABOR:%s\n", TriggerArr[systemToAbort]));      
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: LF Markers Off                                                  */
/* Purpose:  This function switches all LF markers off.                      */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_LFMarkOff (ViSession instrSession)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B5);

    CHECKERR (viPrintf (instrSession, ":SOUR2:MARK:AOFF\n"));     
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Markers Frequency Sweep Off                                     */
/* Purpose:  This function switches all RF frequency markers off.            */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_markersFreqSweepOff (ViSession instrSession)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B4);

    CHECKERR (viPrintf (instrSession, ":SOUR:MARK:FSW:AOFF\n"));      
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Markers Level Sweep Off                                         */
/* Purpose:  This function switches all RF level markers off.                */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_markersLevelSweepOff (ViSession instrSession)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMR_OPT_B4);

    CHECKERR (viPrintf (instrSession, ":SOUR:MARK:PSW:AOFF\n"));      
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}


/*===========================================================================*/
/* Function: Calibration                                                     */
/* Purpose:  This function calibrates selected component.                    */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_calibration (ViSession instrSession, ViInt32 toCal,
                ViInt32 timeout, ViChar _VI_FAR CalResults[])
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    ViInt32 oldTimeout;

    if (rssmr_invalidViInt32Range (toCal, 0, 1))
        return VI_ERROR_PARAMETER2;
    if (rssmr_invalidViInt32Range (timeout, 0, 1800000))
        return VI_ERROR_PARAMETER3;
    if (!CalResults)
        return VI_ERROR_PARAMETER4;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_TMO_VALUE, &oldTimeout));
    CHECKERR (viSetAttribute (instrSession, VI_ATTR_TMO_VALUE, timeout));
    CHECKERR (viQueryf (instrSession, "%s?\n", "%[^\r\n]", CalArr[toCal], CalResults));
    CHECKERR (viSetAttribute (instrSession, VI_ATTR_TMO_VALUE, oldTimeout));

    CHECKERR (rssmr_checkStatus (instrSession));

    return rssmr_status;
}

/*===========================================================================*/
/* Function: System Keylock                                                  */
/* Purpose:  This function sets the local lockout.                           */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_utilSystemKeylock (ViSession instrSession,
                ViBoolean keylock)
{
    ViStatus rssmr_status = VI_SUCCESS;
    
    if (rssmr_invalidViBooleanRange (keylock))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SYST:KLOC %s\n", keylock ? "ON" : "OFF"));
    CHECKERR (rssmr_checkStatus (instrSession));

    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set Display Update                                              */
/* Purpose:  This function switches on or off the update of all instruments  */
/*           display elements.                                               */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_utilSetDisplayUpdate (ViSession instrSession,
                ViBoolean displayUpdate)
{
    ViStatus rssmr_status = VI_SUCCESS;
    
    if (rssmr_invalidViBooleanRange (displayUpdate))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SYST:DISP:UPD %s\n",
        displayUpdate ? "ON" : "OFF"));
    CHECKERR (rssmr_checkStatus (instrSession));

    return rssmr_status;
}

/*===========================================================================*/
/* Function: Recall Setup                                                    */
/* Purpose:  This function recalls setup from memory.                        */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_utilRecallSetup (ViSession instrSession,
                                            ViInt32 registerNo)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    
    if (rssmr_invalidViInt32Range (registerNo, 1, 50))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, "*RCL %Ld\n", registerNo));     
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Save Setup                                                      */
/* Purpose:  This function saves setup to memory.                            */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_utilSaveSetup (ViSession instrSession,
                                          ViInt32 registerNo)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    
    if (rssmr_invalidViInt32Range (registerNo, 1, 50))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, "*SAV %Ld\n", registerNo));     
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Get Free Settings                                               */
/* Purpose:  This function returns the number of free memories.              */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_utilGetFreeSett (ViSession instrSession, ViPInt32 value)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    
    if (!value)  
       return VI_ERROR_PARAMETER2;

    CHECKERR (viQueryf (instrSession, ":MEM:NST?\n", "%ld", value));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set Register                                                    */
/* Purpose:  This function sets value to  selected register.                 */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_utilSetReg (ViSession instrSession,
                                    ViInt32 registerName, ViInt32 value)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    
    if (rssmr_invalidViInt32Range (registerName, 0, 13))     
       return VI_ERROR_PARAMETER2;
    if (registerName > 9) { 
        if (rssmr_invalidViInt32Range (value, 0, 255))   
            return VI_ERROR_PARAMETER3;
    }
    else {
        if (rssmr_invalidViInt32Range (value, 0, 32767))     
            return VI_ERROR_PARAMETER3;
    }

    if ((registerName!=0) && (registerName!=1) && (registerName!=5) &&
        (registerName!=6) && (registerName!=11) && (registerName!=13))
        {
        CHECKERR (viPrintf (instrSession, "%s %ld\n", RegisterArr[registerName],
            value));
        }

    return rssmr_status;
}

/*===========================================================================*/
/* Function: Get Register                                                    */
/* Purpose:  This function returns value from  selected register.            */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_utilGetReg (ViSession instrSession,
                                    ViInt32 registerName, ViPInt32 value)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    
    if (rssmr_invalidViInt32Range (registerName, 0, 13))
       return VI_ERROR_PARAMETER2;
    if (!value)
       return VI_ERROR_PARAMETER3;

    CHECKERR (viQueryf (instrSession, "%s?\n", "%ld", RegisterArr[registerName],
        value));

    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set System Mode                                                 */
/* Purpose:  This function selects system mode.                              */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_utilSetSystemMode (ViSession instrSession, ViInt32 mode)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    
    if (rssmr_invalidViInt32Range (mode, 0, 1))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SYST:MODE %s\n", SystemModeArr[mode]));
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set Display Mode                                                */
/* Purpose:  This function selects mode of the specified display.            */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_utilSetDisplayMode (ViSession instrSession,
                                            ViInt32 parameter, ViBoolean state)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    
    if (rssmr_invalidViInt32Range (parameter, 0, 2))
       return VI_ERROR_PARAMETER2;
    if (rssmr_invalidViBooleanRange (state))
       return VI_ERROR_PARAMETER3;

    CHECKERR (viPrintf (instrSession, ":DISP:ANN:%s %s\n", DisplayParamArr[parameter],
        state ? "ON" : "OFF"));     
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Set GPIB Address                                                */
/* Purpose:  This function sets GPIB addres of the instrument.               */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_utilSetGPIBAddr (ViSession instrSession,
                                         ViInt32 address)
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    
    if (rssmr_invalidViInt32Range (address, 1, 30))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SYST:COMM:GPIB:SELF:ADDR %ld\n", address));       
    CHECKERR (rssmr_checkStatus (instrSession));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Configure RS232.                                                */
/* Purpose:  This function sets the baud rate, data bits, stop bits, hardware*/
/*           and software hadshake and parity                                */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_utilConfigureRS232 (ViSession instrSession,
                ViInt32 baudRate, ViInt32 dataBits, ViInt32 stopBits,
                ViInt32 hardwareHandshake, ViInt32 softwareHandshake,
                ViInt32 parity)
{
    ViStatus rssmr_status = VI_SUCCESS;

    if (rssmr_invalidViInt32Range (baudRate,0,7))
       return VI_ERROR_PARAMETER2;
    if (rssmr_invalidViInt32Range (dataBits, 7, 8))
       return VI_ERROR_PARAMETER3;
    if (rssmr_invalidViInt32Range (stopBits, 1, 2))
       return VI_ERROR_PARAMETER4;
    if (rssmr_invalidViInt32Range (hardwareHandshake, 0, 3))
       return VI_ERROR_PARAMETER5;
    if (rssmr_invalidViInt32Range (softwareHandshake, 0, 1))
       return VI_ERROR_PARAMETER6;
    if (rssmr_invalidViInt32Range (parity, 0, 2))
       return VI_ERROR_PARAMETER7;

    if (hardwareHandshake == 3) /* Hardware handshake does not apply. */
        CHECKERR (viPrintf (instrSession, ":SYST:COMM:SER:BAUD %s;BITS %ld"
            ";SBIT %ld;PACE %s;PAR %s\n", BaudRateArr[baudRate],
            dataBits, stopBits, softHandArr[softwareHandshake], ParityArr[parity]));
    else
        CHECKERR (viPrintf (instrSession, ":SYST:COMM:SER:BAUD %s;BITS %ld"
            ";SBIT %ld;PACE %s;PAR %s;CONT:RTS %s\n", BaudRateArr[baudRate],
            dataBits, stopBits, softHandArr[softwareHandshake], ParityArr[parity],
            hardHandArr[hardwareHandshake]));

    CHECKERR (rssmr_checkStatus (instrSession));

    return rssmr_status;
}

/*===========================================================================*/
/* Function: Test                                                            */
/* Purpose:  This function tests selected component.                         */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_utilTest (ViSession instrSession, ViInt32 toTest,
                                  ViChar _VI_FAR testResults[])
{                               
    ViStatus rssmr_status = VI_SUCCESS;
    
    if (rssmr_invalidViInt32Range (toTest, 0, 2))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viQueryf (instrSession, ":TEST:%s?\n", "%[^\r\n]", TestArr[toTest],
        testResults));
    CHECKERR (rssmr_checkStatus (instrSession));

    return rssmr_status;
}

/*===========================================================================*/
/* Function: Flush Error Queue                                               */
/* Purpose:  This function deletes all error queue.                          */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_FlushErrorQueue (ViSession instrSession)
{
    ViStatus rssmr_status = VI_SUCCESS;
    ViInt32 errorCode = 0;

    do {
        CHECKERR (viPrintf (instrSession, ":STAT:QUE:NEXT?\n"));
        CHECKERR (viScanf (instrSession, "%ld,%*[^\r\n]", &errorCode));
    }
    while (errorCode != 0);

    CHECKERR (rssmr_checkStatus (instrSession));

    return rssmr_status;
}

/*===========================================================================*/

/*****************************************************************************/
/*-------- INSERT USER-CALLABLE INSTRUMENT-DEPENDENT ROUTINES HERE ----------*/
/*****************************************************************************/
           
/*===========================================================================*/
/* Function: State Checking                                                  */
/* Purpose:  This function switches state checking of the instrument (reading*/
/*           of the Standard Event Register and checking it for error) status*/
/*           subsystem. Driver functions are using state checking which is by*/
/*           default enabled.                                                */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_errorCheckState (ViSession instrSession,
                                          ViBoolean state)
{
    ViStatus rssmr_status = VI_SUCCESS;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    instrPtr -> errorChecking = state;

    return rssmr_status;
}

/*===========================================================================*/
/* Function: Write To Instrument                                             */
/* Purpose:  This function writes a command string to the instrument.        */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_writeInstrData (ViSession instrSession, ViString writeBuffer)
{
    ViStatus rssmr_status = VI_SUCCESS;
    
    CHECKERR (viPrintf (instrSession, "%s\n", writeBuffer));
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Read Instrument Buffer                                          */
/* Purpose:  This function reads the output buffer of the instrument.        */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_readInstrData (ViSession instrSession,
                                        ViInt32 numberBytesToRead,
                                        ViChar _VI_FAR readBuffer[],
                                        ViPInt32 numBytesRead)
{
    ViStatus rssmr_status = VI_SUCCESS;
    ViUInt32 tmpCnt;

    if (numBytesRead)
        *numBytesRead = 0L;
        
    CHECKERR (viRead (instrSession, (ViBuf)readBuffer, numberBytesToRead, &tmpCnt));
    
    if (numBytesRead)
        *numBytesRead = (ViInt32)tmpCnt;
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Reset                                                           */
/* Purpose:  This function resets the instrument.  If the reset function     */
/*           is not supported by the instrument, this function returns       */
/*           the warning VI_WARN_NSUP_RESET.                                 */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_reset (ViSession instrSession)
{
    ViStatus rssmr_status = VI_SUCCESS;

    /*  Initialize the instrument to a known state.  */
    CHECKERR (viPrintf (instrSession, "*RST;*CLS\n"));
    CHECKERR (rssmr_defaultInstrSetup (instrSession));
        
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Self-Test                                                       */
/* Purpose:  This function executes the instrument self-test and returns     */
/*           the result. If the self test function is not supported by the   */
/*           instrument, this function returns the warning                   */
/*           VI_WARN_NSUP_SELF_TEST.                                         */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_self_test (ViSession instrSession,
                                    ViPInt32 testResult,
                                    ViChar _VI_FAR testMessage[])
{
    ViStatus rssmr_status = VI_SUCCESS;
    ViChar result1[20], result2[20], result3[20];
    ViBoolean   isPassed;

    CHECKERR (viQueryf (instrSession, ":TEST:RAM?\n", "%s", result1));
    CHECKERR (viQueryf (instrSession, ":TEST:ROM?\n", "%s", result2));
    CHECKERR (viQueryf (instrSession, ":TEST:BATT?\n", "%s", result3));
    
    isPassed = (!strstr (result1, "Passed") ||
                !strstr (result2, "Passed") ||
                !strstr (result3, "O.K."));

    if (testResult)
         *testResult = (!isPassed) ? 0 : (-1);
    
    if (testMessage)
        sprintf(testMessage, "Self-Test %s.",
            (!isPassed) ? "Passed" : "Failed");
    
    CHECKERR (rssmr_checkStatus (instrSession));

    return rssmr_status;
}

/*===========================================================================*/
/* Function: Error Query                                                     */
/* Purpose:  This function queries the instrument error queue, and returns   */
/*           the result. If the error query function is not supported by the */
/*           instrument, this function returns the warning                   */
/*           VI_WARN_NSUP_ERROR_QUERY.                                       */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_error_query (ViSession instrSession,
                                      ViPInt32 errorCode,
                                      ViChar _VI_FAR errorMessage[])
{
    ViStatus rssmr_status = VI_SUCCESS;

    CHECKERR (viPrintf (instrSession, ":SYST:ERR?\n"));
    CHECKERR (viScanf (instrSession, "%ld,\"%[^\"]", errorCode, errorMessage));

    return rssmr_status;
}

/*===========================================================================*/
/* Function: Error Message                                                   */
/* Purpose:  This function translates the error return value from the        */
/*           instrument driver into a user-readable string.                  */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_error_message (ViSession instrSession,
                                        ViStatus statusCode,
                                        ViChar _VI_FAR errMessage[])
{
    ViStatus rssmr_status = VI_SUCCESS;
    ViInt16 i;

    static rssmr_tStringValPair statusDescArray[] = {
        {VI_WARN_NSUP_ID_QUERY,     "WARNING: ID Query not supported"},
        {VI_WARN_NSUP_RESET,        "WARNING: Reset not supported"},
        {VI_WARN_NSUP_SELF_TEST,    "WARNING: Self-test not supported"},
        {VI_WARN_NSUP_ERROR_QUERY,  "WARNING: Error Query not supported"},     
        {VI_WARN_NSUP_REV_QUERY,    "WARNING: Revision Query not supported"},
        {VI_ERROR_PARAMETER1,   "ERROR: Parameter 1 out of range"},
        {VI_ERROR_PARAMETER2,   "ERROR: Parameter 2 out of range"},
        {VI_ERROR_PARAMETER3,   "ERROR: Parameter 3 out of range"},
        {VI_ERROR_PARAMETER4,   "ERROR: Parameter 4 out of range"},
        {VI_ERROR_PARAMETER5,   "ERROR: Parameter 5 out of range"},
        {VI_ERROR_PARAMETER6,   "ERROR: Parameter 6 out of range"},
        {VI_ERROR_PARAMETER7,   "ERROR: Parameter 7 out of range"},
        {VI_ERROR_PARAMETER8,   "ERROR: Parameter 8 out of range"},
        {VI_ERROR_FAIL_ID_QUERY,"ERROR: Identification query failed"},
        {VI_ERROR_INV_RESPONSE, "ERROR: Interpreting instrument response"},
        {VI_ERROR_INSTR_FILE_OPEN,    "ERROR: Opening the specified file"},
        {VI_ERROR_INSTR_FILE_WRITE,   "ERROR: Writing to the specified file"},
        {VI_ERROR_INSTR_INTERPRETING_RESPONSE, "ERROR: Interpreting the instrument's response"},
        {VI_ERROR_INSTR_PARAMETER9 ,  "ERROR: Parameter 9 out of range"},
        {VI_ERROR_INSTR_PARAMETER10,  "ERROR: Parameter 10 out of range"},
        {VI_ERROR_INSTR_PARAMETER11,  "ERROR: Parameter 11 out of range"},
        {VI_ERROR_INSTR_PARAMETER12,  "ERROR: Parameter 12 out of range"},
        {VI_ERROR_INSTR_PARAMETER13,  "ERROR: Parameter 13 out of range"},
        {VI_ERROR_INSTR_PARAMETER14,  "ERROR: Parameter 14 out of range"},
        {VI_ERROR_INSTR_PARAMETER15,  "ERROR: Parameter 15 out of range"},

        {RSSMR_ERROR_INSTRUMENT_ERROR,  "ERROR: Instrument status error"},
        {RSSMR_ERROR_INVALID_CONFIGURATION, "ERROR: Instrument configuration error"},
        {RSSMR_ERROR_INSTRUMENT_OPTION, "ERROR: Option not installed"},
        {VI_NULL, VI_NULL}
    };

    rssmr_status = viStatusDesc (instrSession, statusCode, errMessage);
    if (rssmr_status == VI_WARN_UNKNOWN_STATUS)
        {
        for (i = 0; statusDescArray[i].stringName; i++)
            {
            if (statusDescArray[i].stringVal == statusCode)
                {
                sprintf (errMessage, "%s", statusDescArray[i].stringName);
                return (VI_SUCCESS);
                }
            }
        sprintf (errMessage, "Unknown Error 0x%X", statusCode);
        return (VI_WARN_UNKNOWN_STATUS);
        }
    
    rssmr_status = VI_SUCCESS;
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Revision Query                                                  */
/* Purpose:  This function returns the driver and instrument revisions.      */
/*           If the revision query function is not supported by the          */ 
/*           instrument, this function returns the warning                   */
/*           VI_WARN_NSUP_REV_QUERY.                                         */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_revision_query (ViSession instrSession,
                    ViChar _VI_FAR driverRev[], ViChar _VI_FAR instrRev[])
{
    ViStatus rssmr_status = VI_SUCCESS;

    CHECKERR (viPrintf (instrSession, "*IDN?\n"));
    CHECKERR (viScanf (instrSession, "%*[^,],%*[^,],%*[^,],%[^\r\n]", instrRev));

    sprintf (driverRev, "%s", RSSMR_REVISION);
    
    return rssmr_status;
}

/*===========================================================================*/
/* Function: Close                                                           */
/* Purpose:  This function closes the instrument.                            */
/*===========================================================================*/
ViStatus _VI_FUNC rssmr_close (ViSession instrSession)
{
    rssmr_instrRange instrPtr;
    ViSession rmSession;
    ViStatus rssmr_status = VI_SUCCESS;

    CHECKERR( viGetAttribute (instrSession, VI_ATTR_RM_SESSION, &rmSession));
    CHECKERR( viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    if (instrPtr != NULL) 
        free (instrPtr);

    rssmr_status = viClose (instrSession);
    viClose (rmSession);

    return rssmr_status;
}

/*****************************************************************************/
/*= UTILITY ROUTINES (Non-Exportable Functions) =============================*/
/*****************************************************************************/

/*===========================================================================*/
/* Function: Boolean Value Out Of Range - ViBoolean                          */
/* Purpose:  This function checks a Boolean to see if it is equal to VI_TRUE */
/*           or VI_FALSE. If the value is out of range, the return value is  */
/*           VI_TRUE, otherwise the return value is VI_FALSE.                */
/*===========================================================================*/
ViBoolean rssmr_invalidViBooleanRange (ViBoolean val)
{
    return (ViBoolean)((val != VI_FALSE && val != VI_TRUE) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Long Signed Integer Value Out Of Range - ViInt32                */
/* Purpose:  This function checks a long signed integer value to see if it   */  
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range, the return value is VI_TRUE, otherwise the return     */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean rssmr_invalidViInt32Range (ViInt32 val, ViInt32 min, ViInt32 max)
{
    return (ViBoolean)((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Real (Double) Value Out Of Range - ViReal64                     */
/* Purpose:  This function checks a real (double) value to see if it lies    */  
/*           between a minimum and maximum value.  If the value is out of    */
/*           range, the return value is VI_TRUE, otherwise the return value  */
/*           is VI_FALSE.                                                    */
/*===========================================================================*/
ViBoolean rssmr_invalidViReal64Range (ViReal64 val, ViReal64 min, ViReal64 max)
{
    return (ViBoolean)((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Check String                                                    */
/* Purpose:  This function checks a string value to see if it is             */
/*           of a defined length and initialized.                            */
/*===========================================================================*/
ViBoolean rssmr_invalidViString (ViString val, ViUInt32 maxLength)
{
    if (val)
        return (ViBoolean)((strlen (val) > maxLength) ? VI_TRUE : VI_FALSE);

    return VI_TRUE;
}

/*===========================================================================*/
/* Function: Initialize Clean Up                                             */
/* Purpose:  This function is used only by the rssmr_init function.  When    */
/*           an error is detected this function is called to close the       */
/*           open resource manager and instrument object sessions and to     */
/*           set the instrSession that is returned from rssmr_init to        */
/*           VI_NULL.                                                        */
/*===========================================================================*/
ViStatus rssmr_initCleanUp (ViSession openRMSession,
                             ViPSession openInstrSession,
                             ViStatus currentStatus)
{
    rssmr_instrRange instrPtr;
    
    if (viGetAttribute (*openInstrSession, VI_ATTR_USER_DATA, &instrPtr) >= 0)
        if (instrPtr != NULL) 
            free (instrPtr);

    viClose (*openInstrSession);
    viClose (openRMSession);
    *openInstrSession = VI_NULL;
    
    return currentStatus;
}

/*****************************************************************************/
/*----------- INSERT INSTRUMENT-DEPENDENT UTILITY ROUTINES HERE -------------*/
/*****************************************************************************/

/*===========================================================================*/
/* Function: Default Instrument Setup                                        */
/* Purpose:  This function sends a default setup to the instrument.  This    */
/*           function is called by the rssmr_reset operation and by the      */
/*           rssmr_init function if the reset option has not been            */
/*           selected.  This function is useful for configuring any          */
/*           instrument settings that are required by the rest of the        */
/*           instrument driver functions such as turning headers ON or OFF   */
/*           or using the long or short form for commands, queries, and data.*/                                    
/*===========================================================================*/
ViStatus rssmr_defaultInstrSetup (ViSession instrSession)
{
    ViStatus    rssmr_status = VI_SUCCESS;
    ViChar      instrRev[BUFFER_SIZE],
                instrModel[BUFFER_SIZE];
    rssmr_instrRange instrPtr;

    /* Determine if the structure has been initialized for the current VISA  */
    /* Session and malloc if it has not.                                     */
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    
    if (instrPtr == NULL) 
        instrPtr = malloc (sizeof (struct rssmr_statusDataRanges));

    CHECKERR (rssmr_instrOptions (instrSession, &(instrPtr -> instrOption)));
    /* Presency of option RSSMR_OPT_B1 is temporarily hardcoded due to firmware problem (ver 1.23) */
    instrPtr -> instrOption |= RSSMR_OPT_B1;

    instrPtr -> errorChecking   = VI_TRUE;
    instrPtr -> instrModel      = 0;
    strcpy (instrPtr -> instrDriverRevision, RSSMR_REVISION);

    CHECKERR (viQueryf (instrSession, "*IDN?\n", "%*[^,],%[^,],%*[^,],%[^\r\n]",
                        instrModel, instrRev));

    /* Check for instrument model */
    {
    ViInt32     i = 0;
    ViString    instrList[] =
        {
        "SMR20",
        "SMR27",
        "SMR30",
        "SMR40",
        "SMR50",
        "SMR60",

        VI_NULL
        };

    for ( i = 0; instrList[i]; i++ )
        if (strstr (instrModel, instrList[i]))
            {
            instrPtr -> instrModel = (1 << i);
            break;
            }
    }

    /* VI_FALSE ... Firmware version 1.07 */
    /* VI_TRUE ...  Firmware version 1.08 or higher */
    
    instrPtr -> firmware = (!strstr (instrRev, "1.07")) ? VI_TRUE : VI_FALSE ;

    CHECKERR (viSetAttribute (instrSession, VI_ATTR_USER_DATA, (ViUInt32)instrPtr));

    return rssmr_status;
}

/*=========================================================================*/
/* Function: Check Status                                                  */
/* Purpose:  This function reads the system status byte.                   */
/*=========================================================================*/
ViStatus rssmr_checkStatus (ViSession instrSession)
{
    ViStatus    rssmr_status = VI_SUCCESS;
    ViInt32     esrByte = 0;
    rssmr_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    if (instrPtr -> errorChecking)
        { /* Is Error Checking Enabled? */
        /*--------------- Read the Standard Event Register ------------------*/
        CHECKERR (viQueryf (instrSession, "*ESR?\n", "%ld", &esrByte));
        
        /*---------------- Check if any error or message bit was asserted ---*/
        rssmr_status = (esrByte & IEEE_ERROR_BITS) ? RSSMR_ERROR_INSTRUMENT_ERROR : VI_SUCCESS;
        }

    return rssmr_status;
}

/*===========================================================================*/
/* Function: Instrument Options                                              */
/* Purpose:  This function returns a list of options which the instrument is */
/*           equipped with.                                                  */
/*===========================================================================*/
ViStatus rssmr_instrOptions (ViSession instrSession, ViPInt32 options)
{                              
    ViStatus    rssmr_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    ViChar      option[BUFFER_SIZE],
                *p2option;
    ViUInt32    i;

    ViString optionsList[] =
        {
        "B1",       /* Reference oscillator OXCO */
        "",         /* reserved */
        "B3",       /* Frequency resolution 0.1 Hz */
        "B4",       /* Ramp sweep */
        "B5",       /* AM/FM scan modulator */
        "",         /* reserved */
        "B11",      /* Frequency extension 0.01...1 GHz */
        "B14",      /* Pulse generator */
        "B15",      /* RF attenuator 20 GHz (SMR20/SMR27) */
        "B17",      /* RF attenuator 40 GHz (SMR30/SMR40) */
        "B18",      /* RF attenuator 60 GHz (SMR60) */
        "B19",      /* Rear panel connectors RF - LF (SMR20/SMR27) */
        "B20",      /* Rear panel connectors RF - LF (SMR30/SMR40) */
        "B23",      /* IF input, 20 GHz (SMR20) */
        "B24",      /* IF input, 40 GHz (SMR27/SMR30/SMR40) */

        VI_NULL
        };
    
    CHECKERR (viQueryf (instrSession, "*OPT?\n", "%[^\r\n]", buffer));
  
    *options = 0;        
    
    for (i = 0; optionsList[i]; i++)
        {
        if (strlen(optionsList[i]) == 0)
            continue;

        strcpy(option, optionsList[i]);
        p2option = buffer;

        while ((p2option = strstr(p2option, option)) != VI_NULL)
            {
            /* the last option */
            if (strchr(p2option, ',') && !strchr(option, ','))
                strcat(option, ",");

            if (!strncmp(p2option, option, strlen(option)))
                {
                *options |= (1 << i);
                break;
                }
            p2option++;
            }
        }

    return rssmr_status;
}

/*****************************************************************************/
/*=== END INSTRUMENT DRIVER SOURCE CODE =====================================*/
/*****************************************************************************/
